@@ -25,6 +25,7 @@ protected:
 	FXOption* 			option_z;	
 	FXOption* 			option_rar;	
 	FXOption* 			option_lzh; 	
+	FXOption* 			option_arj; 	
 
 private:
     ArchInputDialog()
@@ -45,6 +46,7 @@ public:
 		ID_FORMAT_Z,
 		ID_FORMAT_RAR,
 		ID_FORMAT_LZH,
+		ID_FORMAT_ARJ,
         ID_LAST
     };
     ArchInputDialog(FXWindow*,FXString);
