@@ -11,12 +11,13 @@
 
 #include "xfedefs.h"
 #include "icons.h"
-#include "DirList.h"
+#include "xfeutils.h"
 #include "File.h"
 #include "FileDict.h"
 #include "InputDialog.h"
 #include "MessageBox.h"
 #include "XFileExplorer.h"
+#include "DirList.h"
 
 
 // Interval between updevices and mtdevices read (s)
@@ -44,10 +45,6 @@ extern FXStringDict* updevices;
 extern FXbool allowPopupScroll;
 
 
-extern FXMainWindow *mainWindow;
-
-
-
 // Object implementation
 FXIMPLEMENT(DirItem,FXTreeItem,NULL,0)
 
@@ -134,8 +131,8 @@ DirList::DirList(FXWindow *focuswin,FXComposite *p,FXObject* tgt,FXSelector sel,
         {
             while((mnt=getmntent(fstab)))
             {
-                if(!::streq(mnt->mnt_type,MNTTYPE_IGNORE) && !::streq(mnt->mnt_type,MNTTYPE_SWAP) 
-					&& !::streq(mnt->mnt_type,"proc") && !::streq(mnt->mnt_dir,"/"))
+                if(!streq(mnt->mnt_type,MNTTYPE_IGNORE) && !streq(mnt->mnt_type,MNTTYPE_SWAP) 
+					&& !streq(mnt->mnt_dir,"/"))
                 {
                     if(!strncmp(mnt->mnt_fsname,"/dev/fd",7))
                         fsdevices->insert(mnt->mnt_dir,"floppy");
@@ -143,9 +140,9 @@ DirList::DirList(FXWindow *focuswin,FXComposite *p,FXObject* tgt,FXSelector sel,
                         fsdevices->insert(mnt->mnt_dir,"cdrom");
                     else if (!strncmp(mnt->mnt_fsname,"/dev/zip",8))
                         fsdevices->insert(mnt->mnt_dir,"zip");
-                    else if (::streq(mnt->mnt_type,"nfs"))
+                    else if (streq(mnt->mnt_type,"nfs"))
                         fsdevices->insert(mnt->mnt_dir,"nfsdisk");
-                    else if (::streq(mnt->mnt_type,"smbfs"))
+                    else if (streq(mnt->mnt_type,"smbfs"))
                         fsdevices->insert(mnt->mnt_dir,"smbdisk");
                     else
                         fsdevices->insert(mnt->mnt_dir,"harddisk");
@@ -164,7 +161,8 @@ DirList::DirList(FXWindow *focuswin,FXComposite *p,FXObject* tgt,FXSelector sel,
             while((mnt=getmntent(mtab)))
 			{
 				// To fix an issue with some Linux distributions
-				if (!::streq(mnt->mnt_dir,"/dev/.static/dev"))
+				FXString mntdir=mnt->mnt_dir;
+				if (mntdir!="/dev/.static/dev" &&   mntdir.rfind(".gvfs",5,mntdir.length())==-1)
 					mtdevices->insert(mnt->mnt_dir,mnt->mnt_type);
 			}
             endmntent(mtab);
@@ -182,7 +180,8 @@ DirList::DirList(FXWindow *focuswin,FXComposite *p,FXObject* tgt,FXSelector sel,
             while((mnt=getmntent(mtab)))
             {
 				// To fix an issue with some Linux distributions
- 				if (!::streq(mnt->mnt_dir,"/dev/.static/dev"))
+				FXString mntdir=mnt->mnt_dir;
+				if (mntdir!="/dev/.static/dev" &&   mntdir.rfind(".gvfs",5,mntdir.length())==-1)
 				{
 					if (lstatmt(mnt->mnt_dir,&statbuf)==-1)
 						mtstate="down";
@@ -200,7 +199,8 @@ DirList::DirList(FXWindow *focuswin,FXComposite *p,FXObject* tgt,FXSelector sel,
 	FXString homelocation=getenv("HOME");
     if(homelocation=="")
         homelocation=ROOTDIR;
-	trashlocation=homelocation+PATHSEPSTRING+TRASHPATH;
+	trashfileslocation=homelocation+PATHSEPSTRING+TRASHFILESPATH;
+	trashinfolocation=homelocation+PATHSEPSTRING+TRASHINFOPATH;
 }
 
 
@@ -343,7 +343,6 @@ long DirList::onDNDMotion(FXObject* sender,FXSelector sel,void* ptr)
     // Dropping list of filenames
     if(offeredDNDType(FROM_DRAGNDROP,urilistType))
     {
-
         // Locate drop place
         item=(TreeItem*)getItemAt(event->win_x,event->win_y);
 
@@ -533,10 +532,10 @@ long DirList::onDNDDrop(FXObject* sender,FXSelector sel,void* ptr)
                 {
                     title=_("Copy ");
                     icon = copy_bigicon;
-                    if (num==1)
+					if (num==1)
                         message=title+source;
-                    else
-                        message=_("Copy ")+FXStringVal(num)+_(" files/folders.\nFrom: ")+sourcedir;
+                    else				
+						title.format(_("Copy %s files/folders.\nFrom: %s"),FXStringVal(num).text(),sourcedir.text());
                 }
                 else if (dropaction==DRAG_MOVE)
                 {
@@ -545,7 +544,7 @@ long DirList::onDNDDrop(FXObject* sender,FXSelector sel,void* ptr)
                     if (num==1)
                         message=title+source;
                     else
-                        message=_("Move ")+FXStringVal(num)+_(" files/folders.\nFrom: ")+sourcedir;
+						title.format(_("Move %s files/folders.\nFrom: %s"),FXStringVal(num).text(),sourcedir.text());
                 }
                 else if ((dropaction==DRAG_LINK) && (num==1))
                 {
@@ -572,20 +571,52 @@ long DirList::onDNDDrop(FXObject* sender,FXSelector sel,void* ptr)
             {
                 // Move file
                 f->create();
-                ret=f->move(source,target);
 
- 				// An error has occurred
-				if (ret==0 && !f->isCancelled)
+				// If target file is located at trash location, also create the corresponding trashinfo file
+				// Do it silently and don't report any error if it fails
+				FXbool use_trash_can=getApp()->reg().readUnsignedEntry("OPTIONS","use_trash_can",TRUE);
+				
+				if (use_trash_can && FXPath::directory(target)==trashfileslocation )
+				{						
+					// Trash files path name
+					FXString trashpathname=createTrashpathname(source,trashfileslocation);
+					
+					// Adjust target name to get the _N suffix if any
+					FXString trashtarget=FXPath::directory(target)+PATHSEPSTRING+FXPath::name(trashpathname);
+
+					// Create trashinfo file
+					createTrashinfo(source,trashpathname,trashfileslocation,trashinfolocation);
+
+					// Move source to trash target
+					ret=f->move(source,trashtarget);
+				}
+
+				// Move source to target
+				else
+					ret=f->move(source,target);
+
+				// If source file is located at trash location, try to also remove the corresponding trashinfo if it exists
+				// Do it silently and don't report any error if it fails
+				if (use_trash_can && ret && (source.left(trashfileslocation.length())==trashfileslocation) )
 				{
-					f->hide();
+					FXString trashinfopathname=trashinfolocation+PATHSEPSTRING+FXPath::name(source)+".trashinfo";
+					::unlink(trashinfopathname.text());
+				}
+
+				// An error has occurred
+				if (ret==0 && !f->isCancelled())
+				{
+					f->hideProgressDialog();
 					MessageBox::error(this,BOX_OK,_("Error"),_("An error has occurred during the move file operation!"));
+					break;
 				}
 
 				// If action is cancelled in progress dialog
-                if (f->isCancelled)
+                if (f->isCancelled())
                 {
-                    f->hide();
+                    f->hideProgressDialog();
                     MessageBox::error(this,BOX_OK,_("Error"),_("Move file operation cancelled!"));
+					break;
                 }
             }
             // Copy the source file
@@ -593,20 +624,44 @@ long DirList::onDNDDrop(FXObject* sender,FXSelector sel,void* ptr)
             {
                 // Copy file
                 f->create();
-                ret=f->copy(source,target);
+
+				// If target file is located at trash location, also create the corresponding trashinfo file
+				// Do it silently and don't report any error if it fails
+				FXbool use_trash_can=getApp()->reg().readUnsignedEntry("OPTIONS","use_trash_can",TRUE);
+				
+				if (use_trash_can && FXPath::directory(target)==trashfileslocation )
+				{						
+					// Trash files path name
+					FXString trashpathname=createTrashpathname(source,trashfileslocation);
+					
+					// Adjust target name to get the _N suffix if any
+					FXString trashtarget=FXPath::directory(target)+PATHSEPSTRING+FXPath::name(trashpathname);
+
+					// Create trashinfo file
+					createTrashinfo(source,trashpathname,trashfileslocation,trashinfolocation);
+
+					// Copy source to trash target
+					ret=f->copy(source,trashtarget);
+				}
+
+				// Copy source to target
+				else
+					ret=f->copy(source,target);
 
 				// An error has occurred
-				if (ret==0 && !f->isCancelled)
+				if (ret==0 && !f->isCancelled())
 				{
-					f->hide();
+					f->hideProgressDialog();
 					MessageBox::error(this,BOX_OK,_("Error"),_("An error has occurred during the copy file operation!"));
+					break;
 				}
 					
                 // If action is cancelled in progress dialog
-                if (f->isCancelled)
+                if (f->isCancelled())
                 {
-                    f->hide();
+                    f->hideProgressDialog();
                     MessageBox::error(this,BOX_OK,_("Error"),_("Copy file operation cancelled!"));
+					break;
                 }
             }
             // Link the source file (no progress dialog in this case)
@@ -623,10 +678,8 @@ long DirList::onDNDDrop(FXObject* sender,FXSelector sel,void* ptr)
         delete f;
         FXFREE(&data);
         
-		// Refresh the DirList and the FilePanel
-		//setDirectory(targetdir,TRUE);
-		//((XFileExplorer*) mainWindow)->lpanel->current->list->setDirectory(targetdir);
-		//((XFileExplorer*) mainWindow)->lpanel->current->updatePathLinker();
+		// Force a refresh of the DirList
+		onCmdRefresh(0,0,0);
 
 		return 1;
     }
@@ -681,7 +734,7 @@ long DirList::onBeginDrag(FXObject* sender,FXSelector sel,void* ptr)
             {
                 if(!dragfiles.empty())
                     dragfiles+="\r\n";
-                dragfiles+=FXURL::fileToURL(getItemPathname(item));
+                dragfiles+=::fileToURI(getItemPathname(item));
             }
             if(item->first)
                 item=(TreeItem*)item->first;
@@ -1027,7 +1080,8 @@ long DirList::onMtdevicesRefresh(FXObject*,FXSelector,void*)
 			while((mnt=getmntent(mtab)))
 			{
 				// To fix an issue with some Linux distributions
-				if (!::streq(mnt->mnt_dir,"/dev/.static/dev"))
+				FXString mntdir=mnt->mnt_dir;
+				if (mntdir!="/dev/.static/dev" &&   mntdir.rfind(".gvfs",5,mntdir.length())==-1)
 				{
 					tmpdict->insert(mnt->mnt_dir,"");
 					if (mtdevices->find(mnt->mnt_dir))
@@ -1074,7 +1128,8 @@ long DirList::onUpdevicesRefresh(FXObject*,FXSelector,void*)
         while((mnt=getmntent(mtab)))
         {			
 			// To fix an issue with some Linux distributions
-			if (!::streq(mnt->mnt_dir,"/dev/.static/dev"))
+			FXString mntdir=mnt->mnt_dir;
+			if (mntdir!="/dev/.static/dev" &&   mntdir.rfind(".gvfs",5,mntdir.length())==-1)
 			{
 				tmpdict->insert(mnt->mnt_dir,"");
 
@@ -1319,8 +1374,8 @@ void DirList::listChildItems(DirItem *par)
             // If it is not a directory, and not showing files and matching pattern skip it
             if(!S_ISDIR(info.st_mode) && !((options&DIRLIST_SHOWFILES) && FXPath::match(pattern,name,matchmode)))
                 continue;
-
-            // Find it, and take it out from the old list if found
+		   
+		    // Find it, and take it out from the old list if found
             for(pp=po; (item=*pp)!=NULL; pp=&item->link)
             {
                 if(compare(item->label,name)==0)
@@ -1414,7 +1469,7 @@ fnd:
             {
                 if(!::isReadExecutable(pathname))
                 {
-                    openicon=minifolderopenicon;
+                    openicon=minifolderlockedicon;
                     closedicon=minifolderlockedicon;
                 }
                 else
@@ -1456,6 +1511,9 @@ fnd:
             item->assoc=fileassoc;
             item->date=info.st_mtime;
 
+			// Set the HASITEMS flag			
+			(hasSubDirs(pathname.text())==1 ? item->setHasItems(TRUE) : item->setHasItems(FALSE));
+
 			// Default folder type
 			type=_("Folder");
 
@@ -1475,7 +1533,7 @@ fnd:
 			// If we are in trash can, obtain the deletion time
 			deldate=0;
 			del="";
-			if (FXPath::directory(pathname)==trashlocation)
+			if (FXPath::directory(pathname)==trashfileslocation)
 			{
 				char *endptr;
 				FXString str;
@@ -1487,37 +1545,54 @@ fnd:
 			}
 
 #if defined(linux)
-            // Devices have a specific icon
+			// Mounted devices may have a specific icon
+			if(mtdevices->find(pathname.text()))
+			{
+				type=_("Mount point");
+				
+				if(streq(mtdevices->find(pathname.text()),"cifs"))
+				{
+					item->closedIcon=nfsdriveicon;
+					item->openIcon=nfsdriveicon;
+				}
+				else
+				{
+					item->closedIcon=harddiskicon;
+					item->openIcon=harddiskicon;
+				}
+			}
+
+            // Devices found in fstab may have a specific icon
             if(fsdevices->find(pathname.text()))
             {
 				type=_("Mount point");
 				
-                if(::streq(fsdevices->find(pathname.text()),"harddisk"))
+                if(streq(fsdevices->find(pathname.text()),"harddisk"))
                 {
                     item->closedIcon=harddiskicon;
                     item->openIcon=harddiskicon;
                 }
-                else if(::streq(fsdevices->find(pathname.text()),"nfsdisk"))
+                else if(streq(fsdevices->find(pathname.text()),"nfsdisk"))
                 {
                     item->closedIcon=nfsdriveicon;
                     item->openIcon=nfsdriveicon;
                 }
-                else if(::streq(fsdevices->find(pathname.text()),"smbdisk"))
+                else if(streq(fsdevices->find(pathname.text()),"smbdisk"))
                 {
                     item->closedIcon=nfsdriveicon;
                     item->openIcon=nfsdriveicon;
                 }
-                else if(::streq(fsdevices->find(pathname.text()),"floppy"))
+                else if(streq(fsdevices->find(pathname.text()),"floppy"))
                 {
                     item->closedIcon=floppyicon;
                     item->openIcon=floppyicon;
                 }
-                else if(::streq(fsdevices->find(pathname.text()),"cdrom"))
+                else if(streq(fsdevices->find(pathname.text()),"cdrom"))
                 {
                     item->closedIcon=cdromicon;
                     item->openIcon=cdromicon;
                 }
-                else if(::streq(fsdevices->find(pathname.text()),"zip"))
+                else if(streq(fsdevices->find(pathname.text()),"zip"))
                 {
                     item->closedIcon=zipicon;
                     item->openIcon=zipicon;
@@ -1694,15 +1769,15 @@ void DirList::setDirectory(const FXString& pathname,FXbool notify)
     if(!pathname.empty())
     {
         FXString path=FXPath::absolute(getItemPathname((TreeItem*)currentitem),pathname);
-        while(!FXPath::isTopDirectory(path) && !::isDirectory(path))
-        {
+        	
+		while(!FXPath::isTopDirectory(path) && !::isDirectory(path))
             path=FXPath::upLevel(path);
-        }
+
         TreeItem *item=getPathnameItem(path);
         if(id())
             layout();
-        makeItemVisible(item);
-        setCurrentItem(item,notify);
+		makeItemVisible(item);
+		setCurrentItem(item,notify);
     }
 }
 
@@ -1734,7 +1809,7 @@ void DirList::setCurrentFile(const FXString& pathname,FXbool notify)
         TreeItem *item=getPathnameItem(path);
         if(id())
             layout();
-        makeItemVisible(item);
+		makeItemVisible(item);
         setCurrentItem(item,notify);
     }
 }
