@@ -106,10 +106,10 @@ protected:
     FXString		pattern;				// Pattern of file names
     FXuint			matchmode;				// File wildcard match mode
     FXuint			counter;				// Refresh counter
-	FXString		trashlocation;          // Location of the trash directory
+	FXString		trashfileslocation;     // Location of the trash files directory
+	FXString		trashinfolocation;      // Location of the trash info directory
 	FXWindow		*focuswindow;			// Window used to test focus
 protected:
-    //DirList();
     DirList()
     {}
     virtual TreeItem* createItem(const FXString& text,FXIcon* oi,FXIcon* ci,void* ptr);
