@@ -7,27 +7,42 @@
 #include "BrowseInputDialog.h"
 
 
+#include <sys/types.h>
+
+
+
+
 class DirPanel : public FXVerticalFrame
 {
     FXDECLARE(DirPanel)
 protected:
-	DirList	*list;
-	FXPacker *statusbar;
-    FXLabel	*status;
-	FXString homelocation;
-	FXString trashlocation;
-	FXString startlocation;
-	FXDragType urilistType;        // Standard uri-list type
-	FXDragType xfelistType;        // Xfe, Gnome and XFCE list type
-	FXDragType kdelistType;        // KDE list type
-	FXDragType utf8Type;           // UTF-8 text type
-	FXbool clipboard_locked;       // Clipboard locked to prevent changes when viewing it
-	InputDialog* newdirdialog;
-	ArchInputDialog* archdialog;
-	BrowseInputDialog* operationdialog;
-	FXbool paste;
-	FXWindow* focuswindow;
-	FXbool ctrlflag;			// Flag to select the right click control menu
+	DirList*	list;
+	FXPacker*	statusbar;
+    FXLabel*	status;
+	FXLabel*	activeicon;
+	FXString 	homelocation;
+	FXString 	trashlocation;
+	FXString 	trashfileslocation;
+	FXString 	trashinfolocation;
+	FXString 	startlocation;
+	FXDragType 	urilistType;        	// Standard uri-list type
+	FXDragType 	xfelistType;        	// Xfe, Gnome and XFCE list type
+	FXDragType 	kdelistType;        	// KDE list type
+	FXDragType 	utf8Type;           	// UTF-8 text type
+	FXbool 		clipboard_locked;       // Clipboard locked to prevent changes when viewing it
+	InputDialog* 		newdirdialog;
+	ArchInputDialog* 	archdialog;
+	BrowseInputDialog* 	operationdialog;
+	FXbool 		paste;
+	FXWindow* 	focuswindow;
+	FXbool 		ctrlflag;				// Flag to select the right click control menu
+	TextLabel*	paneltitle;				// Panel title
+	FXbool		isactive;				// Flag to indicate is panel has keyboard focus
+	FXbool		stopListRefresh;		// To stop refreshing in some cases
+	time_t  	curr_mtime;				// Current directory mtime
+	FXString	curr_dirpath;			// Current directory path
+	FXbool		allowDirsizeRefresh;	// Allow or avoid directory size refresh 
+
 
 public:
 DirPanel(FXWindow *owner,FXComposite *p,FXColor listbackcolor=FXRGB(255,255,255),FXColor listforecolor=FXRGB(0,0,0), FXbool smoothscroll=TRUE,
@@ -38,6 +53,7 @@ DirPanel(FXWindow *owner,FXComposite *p,FXColor listbackcolor=FXRGB(255,255,255)
     ~DirPanel();
     enum {
         ID_FILELIST=FXVerticalFrame::ID_LAST,
+		ID_STOP_LIST_REFRESH_TIMER,
 		ID_EXPANDTREE,
 		ID_TOGGLE_HIDDEN,
 		ID_COLLAPSEDIR,
@@ -52,6 +68,7 @@ DirPanel(FXWindow *owner,FXComposite *p,FXColor listbackcolor=FXRGB(255,255,255)
 		ID_DIR_SYMLINK,
 		ID_DIR_DELETE,
 		ID_DIR_TRASH,
+		ID_DIR_RESTORE,
 		ID_NEW_DIR,
 		ID_XTERM,
 		ID_COPY_CLIPBOARD,
@@ -83,6 +100,7 @@ public:
 	long onCmdDirMan(FXObject*,FXSelector,void*);
 	long onCmdDirDelete(FXObject*,FXSelector,void*);
 	long onCmdDirTrash(FXObject*,FXSelector,void*);
+	long onCmdDirRestore(FXObject*,FXSelector,void*);
 	long onCmdNewDir(FXObject*,FXSelector,void*);
 	long onCmdXTerm(FXObject*,FXSelector,void*);
 	long onCmdCopyCut(FXObject*,FXSelector,void*);
@@ -96,16 +114,25 @@ public:
 	long onUpdUnmount(FXObject*,FXSelector,void*);
 	long onUpdMenu(FXObject*,FXSelector,void*);
 	long onUpdDirTrash(FXObject*,FXSelector,void*);
+	long onUpdDirRestore(FXObject*,FXSelector,void*);
 	long onUpdDirDelete(FXObject*,FXSelector,void*);
 	long onUpdTitle(FXObject*,FXSelector,void*);
 	long onUpdStatus(FXObject*,FXSelector,void*);
 	long onExpand(FXObject*,FXSelector,void*);
 	long onKeyPress(FXObject*,FXSelector,void*);
-	long onUpdDirectory(FXObject*,FXSelector,void*);
+	long onCmdFocus(FXObject*,FXSelector,void*);
+	long onCmdStopListRefreshTimer(FXObject*,FXSelector,void*);
 #if defined(linux)
 	long onCmdMount(FXObject*,FXSelector,void*);
 #endif
 public:
+
+	void setActive();
+	void setInactive();
+	
+	// Toggle dirsize refresh and force refresh if flag is true
+	void setAllowDirsizeRefresh(FXbool flag);
+
 	// Change sort function
 	void setSortFunc(FXTreeListSortFunc func)
 	{
@@ -166,10 +193,10 @@ public:
 		list->setFocus();
 	}
 
-	// Has focus?
-	FXbool hasFocus(void)
+	// Is panel active?
+	FXbool isActive(void)
 	{
-		return list->hasFocus();
+		return isactive;
 	}
 
 	// Force dir panel refresh
@@ -177,6 +204,11 @@ public:
 	{
 		list->onCmdRefresh(0,0,0);
 	}
+	
+	DirList* getList(void)
+	{
+		return list;
+	}
 
 #if defined(linux)
 	// Force devices refresh
