@@ -1,4 +1,4 @@
-// This is adapted from 'adie', a demo text editor found
+// This is adapted from 'adie', a text editor found
 // in the FOX library and written by Jeroen van der Zijp.
 
 #include "config.h"
@@ -7,6 +7,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/stat.h>
+#include <sys/wait.h>
 #include <string.h>
 #include <unistd.h>
 #include <signal.h>
@@ -19,6 +20,8 @@
 
 #include "xfedefs.h"
 #include "icons.h"
+#include "xfeutils.h"
+#include "startupnotification.h"
 #include "FileDialog.h"
 #include "FontDialog.h"
 #include "MessageBox.h"
@@ -27,8 +30,8 @@
 #include "XFileWrite.h"
 
 
-
 extern FXbool allowPopupScroll;
+FXbool save_win_pos;
 
 
 FXIMPLEMENT_ABSTRACT(FXTextCommand,FXCommand,NULL,0)
@@ -207,7 +210,7 @@ Preferences::Preferences(EditWindow *owner):DialogBox(owner,_("XFileWrite Prefer
     FXint tabcols=getApp()->reg().readIntEntry("OPTIONS","tabcols",80);
 	tabsize->setText(FXStringVal(tabcols));
 
-    new FXLabel(matrix,_("Strip carriage returns:               "),NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
+    new FXLabel(matrix,_("Strip carriage returns:")+(FXString)"                ",NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
     stripcr = new FXCheckButton(matrix,FXString::null,NULL,0,LAYOUT_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_ROW,0,0,0,0, 0,0,0,0);
     FXint stripreturn=getApp()->reg().readIntEntry("OPTIONS","stripreturn",FALSE);
 	stripcr->setCheck(stripreturn);
@@ -487,6 +490,7 @@ FXDEFMAP(EditWindow) EditWindowMap[]=
 	FXMAPFUNC(SEL_UPDATE,0,EditWindow::onUpdateTitle),
 	FXMAPFUNC(SEL_FOCUSIN,0,EditWindow::onFocusIn),
 	FXMAPFUNC(SEL_COMMAND,EditWindow::ID_ABOUT,EditWindow::onCmdAbout),
+	FXMAPFUNC(SEL_SIGNAL,EditWindow::ID_HARVEST,EditWindow::onSigHarvest),
 	FXMAPFUNC(SEL_COMMAND,EditWindow::ID_NEW,EditWindow::onCmdNew),
 	FXMAPFUNC(SEL_COMMAND,EditWindow::ID_OPEN,EditWindow::onCmdOpen),
 	FXMAPFUNC(SEL_COMMAND,EditWindow::ID_OPEN_RECENT,EditWindow::onCmdOpenRecent),
@@ -525,7 +529,7 @@ FXIMPLEMENT(EditWindow,FXMainWindow,EditWindowMap,ARRAYNUMBER(EditWindowMap))
 EditWindow::EditWindow(XFileWrite* a,const FXString& file):FXMainWindow(a,"XFileWrite",NULL,NULL,DECOR_ALL),mrufiles(a)
 {
     FXHotKey hotkey;
-    FXButton *btn = NULL;
+	FXString key;
 
     // Add to list of windows
     getApp()->windowlist.append(this);
@@ -550,7 +554,7 @@ EditWindow::EditWindow(XFileWrite* a,const FXString& file):FXMainWindow(a,"XFile
 
     // Toolbar
     dragshell=new FXToolBarShell(this,FRAME_RAISED);
-    toolbar=new FXToolBar(topdock,dragshell,LAYOUT_DOCK_NEXT|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|FRAME_RAISED);
+    toolbar=new FXToolBar(topdock,dragshell,LAYOUT_DOCK_NEXT|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_RAISED);
     new FXToolBarGrip(toolbar,toolbar,FXToolBar::ID_TOOLBARGRIP,TOOLBARGRIP_DOUBLE);
 
     // File menu
@@ -581,13 +585,8 @@ EditWindow::EditWindow(XFileWrite* a,const FXString& file):FXMainWindow(a,"XFile
     helpmenu=new FXMenuPane(this);
     new FXMenuTitle(menubar,_("&Help"),NULL,helpmenu,LAYOUT_LEFT);
 
-    // Splitter
-    FXSplitter* splitter=new FXSplitter(this,LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y|SPLITTER_TRACKING);
-
-    // Sunken border for text widget
-    FXHorizontalFrame *textbox=new FXHorizontalFrame(splitter,FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0, 0,0,0,0);
-
     // Make editor window
+    FXHorizontalFrame *textbox=new FXHorizontalFrame(this,FRAME_NONE|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0, 0,0,0,0);
     editor=new FXText(textbox,this,ID_TEXT,LAYOUT_FILL_X|LAYOUT_FILL_Y|TEXT_SHOWACTIVE);
     editor->setHiliteMatchTime(2000);
     editor->setTextStyle(editor->getTextStyle()|TEXT_FIXEDWRAP);
@@ -623,69 +622,118 @@ EditWindow::EditWindow(XFileWrite* a,const FXString& file):FXMainWindow(a,"XFile
     new FXLabel(statusbar,_("  Line:"),NULL,LAYOUT_RIGHT|LAYOUT_CENTER_Y);
 
     // Toolbar buttons: File manipulation
-    new FXButton(toolbar,_("\tNew (Ctrl-N)\tCreate new document. (Ctrl-N)"),newfileicon,this,ID_NEW,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
-    new FXButton(toolbar,_("\tOpen (Ctrl-O)\tOpen document file. (Ctrl-O)"),fileopenicon,this,ID_OPEN,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
-    new FXButton(toolbar,_("\tSave (Ctrl-S)\tSave document. (Ctrl-S)"),savefileicon,this,ID_SAVE,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
-    new FXButton(toolbar,_("\tClose (Ctrl-W)\tClose document file. (Ctrl-W)"),closefileicon,this,ID_CLOSE,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","new","Ctrl-N");
+    new FXButton(toolbar,TAB+_("New")+PARS(key)+TAB+_("Create new document.")+PARS(key),newfileicon,this,ID_NEW,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
+
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","open","Ctrl-O");
+    new FXButton(toolbar,TAB+_("Open")+PARS(key)+TAB+_("Open document file.")+PARS(key),fileopenicon,this,ID_OPEN,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
+
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","save","Ctrl-S");
+    new FXButton(toolbar,TAB+_("Save")+PARS(key)+TAB+_("Save document.")+PARS(key),savefileicon,this,ID_SAVE,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
+
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","close","Ctrl-W");
+    new FXButton(toolbar,TAB+_("Close")+PARS(key)+TAB+_("Close document file.")+PARS(key),closefileicon,this,ID_CLOSE,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
 
     // Spacer
     new FXSeparator(toolbar,SEPARATOR_GROOVE);
 
-    // Toolbar buttons: Print
-    new FXButton(toolbar,_("\tPrint (Ctrl-P)\tPrint document. (Ctrl-P)"),printicon,this,ID_PRINT,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
-    btn=new FXButton(toolbar,_("\tQuit (Ctrl-Q)\tQuit X File Write. (Ctrl-Q)"),quiticon,getApp(),XFileWrite::ID_CLOSEALL,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
-    btn->hide();
-    hotkey=(CONTROLMASK<<16) | KEY_w;
-    btn->addHotKey(hotkey);
+    // Toolbar buttons: Print and Quit
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","print","Ctrl-P");
+    new FXButton(toolbar,TAB+_("Print")+PARS(key)+TAB+_("Print document.")+PARS(key),printicon,this,ID_PRINT,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
+ 
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","quit","Ctrl-Q");
+    new FXButton(toolbar,TAB+_("Quit")+PARS(key)+TAB+_("Quit X File Write.")+PARS(key),quiticon,getApp(),XFileWrite::ID_CLOSEALL,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
 
     // Spacer
     new FXSeparator(toolbar,SEPARATOR_GROOVE);
 
     // Toolbar buttons: Editing
-    new FXButton(toolbar,_("\tCopy (Ctrl-C)\tCopy selection to clipboard. (Ctrl-C)"),copy_clpicon,editor,FXText::ID_COPY_SEL,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
-    new FXButton(toolbar,_("\tCut (Ctrl-X)\tCut selection to clipboard. (Ctrl-X)"),cut_clpicon,editor,FXText::ID_CUT_SEL,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
-    new FXButton(toolbar,_("\tPaste (Ctrl-V)\tPaste clipboard. (Ctrl-V)"),paste_clpicon,editor,FXText::ID_PASTE_SEL,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","copy","Ctrl-C");
+    new FXButton(toolbar,TAB+_("Copy")+PARS(key)+TAB+_("Copy selection to clipboard.")+PARS(key),copy_clpicon,editor,FXText::ID_COPY_SEL,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
+
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","cut","Ctrl-X");
+    new FXButton(toolbar,TAB+_("Cut")+PARS(key)+TAB+_("Cut selection to clipboard.")+PARS(key),cut_clpicon,editor,FXText::ID_CUT_SEL,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
+
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","paste","Ctrl-V");
+    new FXButton(toolbar,TAB+_("Paste")+PARS(key)+TAB+_("Paste clipboard.")+PARS(key),paste_clpicon,editor,FXText::ID_PASTE_SEL,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
 
     // Spacer
     new FXSeparator(toolbar,SEPARATOR_GROOVE);
 
     // Goto line
-    new FXButton(toolbar,_("\tGoto line (Ctrl-L)\tGoto line number. (Ctrl-L)"),gotolineicon,this,EditWindow::ID_GOTO_LINE,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","goto_line","Ctrl-L");
+    new FXButton(toolbar,TAB+_("Goto line")+PARS(key)+TAB+_("Goto line number."),gotolineicon,this,EditWindow::ID_GOTO_LINE,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
 
     // Spacer
     new FXSeparator(toolbar,SEPARATOR_GROOVE);
 
     // Undo/redo
-    new FXButton(toolbar,_("\tUndo (Ctrl-Z)\tUndo last change. (Ctrl-Z)"),undoicon,&undolist,FXUndoList::ID_UNDO,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
-    new FXButton(toolbar,_("\tRedo (Ctrl-Y)\tRedo last undo. (Ctrl-Y)"),redoicon,&undolist,FXUndoList::ID_REDO,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","undo","Ctrl-Z");
+    new FXButton(toolbar,TAB+_("Undo")+PARS(key)+TAB+_("Undo last change.")+PARS(key),undoicon,&undolist,FXUndoList::ID_UNDO,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
+
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","redo","Ctrl-Y");
+    new FXButton(toolbar,TAB+_("Redo")+PARS(key)+TAB+_("Redo last undo.")+PARS(key),redoicon,&undolist,FXUndoList::ID_REDO,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
 
     // Spacer
     new FXSeparator(toolbar,SEPARATOR_GROOVE);
 
     // Search
-    new FXButton(toolbar,_("\tSearch (Ctrl-F)\tSearch text. (Ctrl-F)"),searchicon,this,EditWindow::ID_SEARCH,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
-    btn=new FXButton(toolbar,_("\tSearch selection backward (Shift-Ctrl-G, Shift-F3)\tSearch backward for selected text. (Shift-Ctrl-G, Shift-F3)"),searchprevicon,this,EditWindow::ID_SEARCH_BACK_SEL,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
-    hotkey=(0001<<16) | KEY_F3;
-    btn->addHotKey(hotkey);
-    btn=new FXButton(toolbar,_("\tSearch selection forward (Ctrl-G, F3)\tSearch forward for selected text. (Ctrl-G, F3)"),searchnexticon,this,EditWindow::ID_SEARCH_FORW_SEL,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
-    btn->addHotKey(KEY_F3);
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","search","Ctrl-F");
+    new FXButton(toolbar,TAB+_("Search")+PARS(key)+TAB+_("Search text.")+PARS(key),searchicon,this,EditWindow::ID_SEARCH,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
+
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","search_prev","Shift-Ctrl-G");
+    new FXButton(toolbar,TAB+_("Search selection backward")+PARS(key)+TAB+_("Search backward for selected text.")+PARS(key),searchprevicon,this,EditWindow::ID_SEARCH_BACK_SEL,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
+
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","search_next","Ctrl-G");
+    new FXButton(toolbar,TAB+_("Search selection forward")+PARS(key)+TAB+_("Search forward for selected text.")+PARS(key),searchnexticon,this,EditWindow::ID_SEARCH_FORW_SEL,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
 
     // Spacer
     new FXSeparator(toolbar,SEPARATOR_GROOVE);
 
     // Preferences
-    new FXToggleButton(toolbar,_("\tWord wrap on (Ctrl-K)\tWord wrap on. (Ctrl-K)"),_("\tWord wrap off (Ctrl-K)\tWord wrap off. (Ctrl-K)"),wrapofficon,wraponicon,this,ID_TOGGLE_WRAP,BUTTON_TOOLBAR|LAYOUT_LEFT|ICON_BEFORE_TEXT);
-    new FXToggleButton(toolbar,_("\tShow line numbers (Ctrl-T)\tShow line numbers. (Ctrl-T)"),_("\tHide line numbers (Ctrl-T)\tHide line numbers. (Ctrl-T)"),hidenumbersicon,shownumbersicon,this,ID_TOGGLE_LINES_NUM,BUTTON_TOOLBAR|LAYOUT_LEFT|ICON_BEFORE_TEXT);
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","word_wrap","Ctrl-K");
+    new FXToggleButton(toolbar,TAB+_("Word wrap on")+PARS(key)+TAB+_("Set word wrap on.")+PARS(key),TAB+_("Word wrap off")+PARS(key)+TAB+_("Set word wrap off.")+PARS(key),wrapofficon,wraponicon,this,ID_TOGGLE_WRAP,BUTTON_TOOLBAR|LAYOUT_LEFT|ICON_BEFORE_TEXT);
+
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","line_numbers","Ctrl-T");
+    new FXToggleButton(toolbar,TAB+_("Show line numbers")+PARS(key)+TAB+_("Show line numbers.")+PARS(key),TAB+_("Hide line numbers")+PARS(key)+TAB+_("Hide line numbers.")+PARS(key),hidenumbersicon,shownumbersicon,this,ID_TOGGLE_LINES_NUM,BUTTON_TOOLBAR|LAYOUT_LEFT|ICON_BEFORE_TEXT);
 
 
     // File Menu entries
-    new FXMenuCommand(filemenu,_("&New\tCtrl-N\tCreate new document. (Ctrl-N)"),newfileicon,this,ID_NEW);
-    new FXMenuCommand(filemenu,_("&Open...\tCtrl-O\tOpen document file. (Ctrl-O)"),fileopenicon,this,ID_OPEN);
-    new FXMenuCommand(filemenu,_("&Save\tCtrl-S\tSave changes to file. (Ctrl-S)"),savefileicon,this,ID_SAVE);
-    new FXMenuCommand(filemenu,_("Save &As...\t\tSave document to another file."),saveasicon,this,ID_SAVEAS);
-    new FXMenuCommand(filemenu,_("&Close\tCtrl-W\tClose document. (Ctrl-W)"),closefileicon,this,ID_CLOSE);
+	FXMenuCommand* mc = NULL;
+	FXString text;
+
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","new","Ctrl-N");
+	text=_("&New...")+TABS(key)+_("Create new document.")+PARS(key);
+    mc=new FXMenuCommand(filemenu,text,newfileicon,this,ID_NEW);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
+
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","open","Ctrl-O");
+	text=_("&Open...")+TABS(key)+_("Open document file.")+PARS(key);
+    mc=new FXMenuCommand(filemenu,text,fileopenicon,this,ID_OPEN);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
+
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","save","Ctrl-S");
+	text=_("&Save")+TABS(key)+_("Save changes to file.")+PARS(key);
+    mc=new FXMenuCommand(filemenu,text,savefileicon,this,ID_SAVE);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
+
+    new FXMenuCommand(filemenu,_("Save &As...")+TAB2+_("Save document to another file."),saveasicon,this,ID_SAVEAS);
+
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","close","Ctrl-W");
+	text=_("&Close")+TABS(key)+_("Close document.")+PARS(key);
+    mc=new FXMenuCommand(filemenu,text,closefileicon,this,ID_CLOSE);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
+
     new FXMenuSeparator(filemenu);
-    new FXMenuCommand(filemenu,_("&Print...\tCtrl-P\tPrint document. (Ctrl-P)"),printicon,this,ID_PRINT);
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","print","Ctrl-P");
+	text=_("&Print...")+TABS(key)+_("Print document.")+PARS(key);
+    mc=new FXMenuCommand(filemenu,text,printicon,this,ID_PRINT);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
 
     // Recent file menu; this automatically hides if there are no files
     FXMenuSeparator* sep1=new FXMenuSeparator(filemenu);
@@ -700,20 +748,65 @@ EditWindow::EditWindow(XFileWrite* a,const FXString& file):FXMainWindow(a,"XFile
     FXMenuSeparator* sep2=new FXMenuSeparator(filemenu);
     sep2->setTarget(&mrufiles);
     sep2->setSelector(FXRecentFiles::ID_ANYFILES);
-    new FXMenuCommand(filemenu,_("&Quit\tCtrl-Q\tQuit X File Write. (Ctrl-Q)"),quiticon,getApp(),XFileWrite::ID_CLOSEALL);
+
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","quit","Ctrl-Q");
+	text=_("&Quit")+TABS(key)+_("Quit X File Write.")+PARS(key);
+    mc=new FXMenuCommand(filemenu,text,quiticon,getApp(),XFileWrite::ID_CLOSEALL);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
 
     // Edit Menu entries
-    new FXMenuCommand(editmenu,_("&Undo\tCtrl-Z\tUndo last change. (Ctrl-Z)"),undoicon,&undolist,FXUndoList::ID_UNDO);
-    new FXMenuCommand(editmenu,_("&Redo\tCtrl-Y\tRedo last undo. (Ctrl-Y)"),redoicon,&undolist,FXUndoList::ID_REDO);
-    new FXMenuCommand(editmenu,_("Revert to &saved\t\tRevert to saved."),reverticon,&undolist,FXUndoList::ID_REVERT);
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","undo","Ctrl-Z");
+	text=_("&Undo")+TABS(key)+_("Undo last change.")+PARS(key);
+    mc=new FXMenuCommand(editmenu,text,undoicon,&undolist,FXUndoList::ID_UNDO);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
+
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","redo","Ctrl-Y");
+	text=_("&Redo")+TABS(key)+_("Redo last undo.")+PARS(key);
+    mc=new FXMenuCommand(editmenu,text,redoicon,&undolist,FXUndoList::ID_REDO);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
+
+    new FXMenuCommand(editmenu,_("Revert to &saved")+TAB2+_("Revert to saved document."),reverticon,&undolist,FXUndoList::ID_REVERT);
+
     new FXMenuSeparator(editmenu);
-    new FXMenuCommand(editmenu,_("&Copy\tCtrl-C\tCopy selection to clipboard. (Ctrl-C)"),copy_clpicon,editor,FXText::ID_COPY_SEL);
-    new FXMenuCommand(editmenu,_("Cu&t\tCtrl-X\tCut selection to clipboard. (Ctrl-X)"),cut_clpicon,editor,FXText::ID_CUT_SEL);
-    new FXMenuCommand(editmenu,_("&Paste\tCtrl-V\tPaste from clipboard. (Ctrl-V)"),paste_clpicon,editor,FXText::ID_PASTE_SEL);
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","copy","Ctrl-C");
+	text=_("&Copy")+TABS(key)+_("Copy selection to clipboard.")+PARS(key);
+    mc=new FXMenuCommand(editmenu,text,copy_clpicon,editor,FXText::ID_COPY_SEL);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
+
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","cut","Ctrl-X");
+	text=_("Cu&t")+TABS(key)+_("Cut selection to clipboard.")+PARS(key);
+    mc=new FXMenuCommand(editmenu,text,cut_clpicon,editor,FXText::ID_CUT_SEL);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
+
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","paste","Ctrl-V");
+	text=_("&Paste")+TABS(key)+_("Paste from clipboard.")+PARS(key);
+    mc=new FXMenuCommand(editmenu,text,paste_clpicon,editor,FXText::ID_PASTE_SEL);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
+
     new FXMenuSeparator(editmenu);
-    new FXMenuCommand(editmenu,_("Lo&wer-case\tCtrl-U\tChange to lower case. (Ctrl-U)"),lowercaseicon,editor,FXText::ID_LOWER_CASE);
-    new FXMenuCommand(editmenu,_("Upp&er-case\tShift-Ctrl-U\tChange to upper case. (Shift-Ctrl-U)"),uppercaseicon,editor,FXText::ID_UPPER_CASE);
-    new FXMenuCommand(editmenu,_("&Goto line...\tCtrl-L\tGoto line number. (Ctrl-L)"),gotolineicon,this,EditWindow::ID_GOTO_LINE);
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","lower_case","Ctrl-U");
+	text=_("Lo&wer-case")+TABS(key)+_("Change to lower case.")+PARS(key);
+    mc=new FXMenuCommand(editmenu,text,lowercaseicon,editor,FXText::ID_LOWER_CASE);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
+
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","upper_case","Shift-Ctrl-U");
+	text=_("Upp&er-case")+TABS(key)+_("Change to upper case.")+PARS(key);
+    mc=new FXMenuCommand(editmenu,text,uppercaseicon,editor,FXText::ID_UPPER_CASE);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
+
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","goto_line","Ctrl-L");
+	text=_("&Goto line...")+TABS(key)+_("Goto line number.")+PARS(key);
+    mc=new FXMenuCommand(editmenu,text,gotolineicon,this,EditWindow::ID_GOTO_LINE);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
 
     // Right mouse popup
     popupmenu=new FXMenuPane(this);
@@ -727,25 +820,52 @@ EditWindow::EditWindow(XFileWrite* a,const FXString& file):FXMainWindow(a,"XFile
     new FXMenuSeparator(popupmenu);
 
     // Search Menu entries
-    new FXMenuCommand(searchmenu,_("&Search...\tCtrl-F\tSearch for a string. (Ctrl-F)"),searchicon,this,EditWindow::ID_SEARCH);
-    new FXMenuCommand(searchmenu,_("&Replace...\tCtrl-R\tSearch for a string and replace with another. (Ctrl-R)"),replaceicon,this,EditWindow::ID_REPLACE);
-    new FXMenuCommand(searchmenu,_("Search sel. &backward\tShift-Ctrl-G\tSearch backward for selected text. (Shift-Ctrl-G, Shift-F3)"),searchprevicon,this,EditWindow::ID_SEARCH_BACK_SEL);
-    new FXMenuCommand(searchmenu,_("Search sel. &forward\tCtrl-G\tSearch forward for selected text. (Ctrl-G, F3)"),searchnexticon,this,EditWindow::ID_SEARCH_FORW_SEL);
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","search","Ctrl-F");
+	text=_("&Search...")+TABS(key)+_("Search for a string.")+PARS(key);
+    mc=new FXMenuCommand(searchmenu,text,searchicon,this,EditWindow::ID_SEARCH);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
+
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","replace","Ctrl-R");
+	text=_("&Replace...")+TABS(key)+_("Search for a string and replace with another.")+PARS(key);
+    mc=new FXMenuCommand(searchmenu,text,replaceicon,this,EditWindow::ID_REPLACE);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
+
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","search_prev","Shift-Ctrl-G");
+	text=_("Search sel. &backward")+TABS(key)+_("Search backward for selected text.")+PARS(key);
+    mc=new FXMenuCommand(searchmenu,text,searchprevicon,this,EditWindow::ID_SEARCH_BACK_SEL);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
+    
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","search_next","Ctrl-G");
+	text=_("Search sel. &forward")+TABS(key)+_("Search forward for selected text.")+PARS(key);
+	mc=new FXMenuCommand(searchmenu,text,searchnexticon,this,EditWindow::ID_SEARCH_FORW_SEL);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
 
 
     // Preferences menu
-    new FXMenuCheck(prefsmenu,_("&Word wrap\tCtrl-K\tToggle word wrap mode. (Ctrl-K)"),this,ID_TOGGLE_WRAP);
-    new FXMenuCheck(prefsmenu,_("&Lines numbering\tCtrl-T\tToggle lines numbering mode. (Ctrl-T)"),this,ID_TOGGLE_LINES_NUM);
-    new FXMenuCheck(prefsmenu,_("&Overstrike\t\tToggle overstrike mode."),editor,FXText::ID_TOGGLE_OVERSTRIKE);
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","word_wrap","Ctrl-K");
+	text=_("&Word wrap")+TABS(key)+_("Toggle word wrap mode.")+PARS(key);
+    mc=new FXMenuCheck(prefsmenu,text,this,ID_TOGGLE_WRAP);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
+
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","line_numbers","Ctrl-T");
+	text=_("&Line numbers")+TABS(key)+_("Toggle line numbers mode.")+PARS(key);
+    mc=new FXMenuCheck(prefsmenu,text,this,ID_TOGGLE_LINES_NUM);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
+
+    new FXMenuCheck(prefsmenu,_("&Overstrike")+TAB2+_("Toggle overstrike mode."),editor,FXText::ID_TOGGLE_OVERSTRIKE);
     new FXMenuSeparator(prefsmenu);
-    new FXMenuCommand(prefsmenu,_("&Font...\t\tChange text font."),fontsicon,this,ID_FONT);
-    new FXMenuCommand(prefsmenu,_("&More preferences...\t\tChange other options."),prefsicon,this,ID_PREFERENCES);
-    //new FXMenuSeparator(prefsmenu);
-    //new FXMenuCommand(prefsmenu,_("&Save Settings\t\tSave settings now."),NULL,this,ID_SAVE_OPTIONS);
+    new FXMenuCommand(prefsmenu,_("&Font...")+TAB2+_("Change text font."),fontsicon,this,ID_FONT);
+    new FXMenuCommand(prefsmenu,_("&More preferences...")+TAB2+_("Change other options."),prefsicon,this,ID_PREFERENCES);
 
     // View Menu entries
-    new FXMenuCheck(viewmenu,_("&Toolbar\t\tDisplay toolbar."),toolbar,FXWindow::ID_TOGGLESHOWN);
-    new FXMenuCheck(viewmenu,_("&Status line\t\tDisplay status line."),statusbar,FXWindow::ID_TOGGLESHOWN);
+    new FXMenuCheck(viewmenu,_("&Toolbar")+TAB2+_("Display toolbar."),toolbar,FXWindow::ID_TOGGLESHOWN);
+    new FXMenuCheck(viewmenu,_("&Status line")+TAB2+_("Display status line."),statusbar,FXWindow::ID_TOGGLESHOWN);
 
     // Window menu
     new FXMenuRadio(windowmenu,FXString::null,this,ID_WINDOW_1);
@@ -800,7 +920,11 @@ EditWindow::EditWindow(XFileWrite* a,const FXString& file):FXMainWindow(a,"XFile
     new FXMenuRadio(windowmenu,FXString::null,this,ID_WINDOW_50);
 
     // Help Menu entries
-    new FXMenuCommand(helpmenu,_("&About X File Write...\tF1\tAbout X File Write. (F1)"),NULL,this,ID_ABOUT,0);
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","help","F1");
+	text=_("&About X File Write...")+TABS(key)+_("About X File Write.")+PARS(key);
+    mc=new FXMenuCommand(helpmenu,text,NULL,this,ID_ABOUT,0);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
 
     // Dialogs
     printdialog=NULL;
@@ -808,10 +932,6 @@ EditWindow::EditWindow(XFileWrite* a,const FXString& file):FXMainWindow(a,"XFile
     searchdialog=NULL;
     replacedialog=NULL;
 
-    // Add some alternative accelerators
-    if(getAccelTable())
-        getAccelTable()->addAccel(MKUINT(KEY_Z,CONTROLMASK|SHIFTMASK),&undolist,FXSEL(SEL_COMMAND,FXUndoList::ID_REDO));
-
     // Recent files
     mrufiles.setTarget(this);
     mrufiles.setSelector(ID_OPEN_RECENT);
@@ -853,6 +973,10 @@ void EditWindow::create()
         urilistType=getApp()->registerDragType(urilistTypeName);
     show(PLACEMENT_DEFAULT);
     editor->setFocus();
+
+#ifdef STARTUP_NOTIFICATION
+	startup_completed();
+#endif
 }
 
 
@@ -1110,13 +1234,20 @@ void EditWindow::loadConfig()
         editor->setFont(font);
     }
 
+    // Read the Xfe registry
+	FXRegistry* reg_xfe=new FXRegistry(XFEAPPNAME,"");
+	reg_xfe->read();
+	
+    // Retain window position flag
+	save_win_pos=reg_xfe->readUnsignedEntry("SETTINGS","save_win_pos",FALSE);
+
+	delete reg_xfe;
+
     // Get size and position from registry
 	if (fromreg)
 	{
 		ww=getApp()->reg().readUnsignedEntry("OPTIONS","width",DEFAULT_WINDOW_WIDTH);
 		hh=getApp()->reg().readUnsignedEntry("OPTIONS","height",DEFAULT_WINDOW_HEIGHT);
-		xx=getApp()->reg().readIntEntry("OPTIONS","xpos",DEFAULT_WINDOW_XPOS);
-		yy=getApp()->reg().readIntEntry("OPTIONS","ypos",DEFAULT_WINDOW_YPOS);
 	}
 
     // Showing the status line?
@@ -1172,8 +1303,15 @@ void EditWindow::loadConfig()
     // Highlight match time
     editor->setHiliteMatchTime(hilitematchtime);
 
-    // Reposition window
-    position(xx,yy,ww,hh);
+	// Get position and position window
+    if (save_win_pos && fromreg)
+	{
+		FXint xpos=getApp()->reg().readIntEntry("OPTIONS","xpos",DEFAULT_WINDOW_XPOS);
+		FXint ypos=getApp()->reg().readIntEntry("OPTIONS","ypos",DEFAULT_WINDOW_YPOS);
+		position(xpos,ypos,ww,hh);
+	}
+	else
+    	position(getX(),getY(),ww,hh);
 }
 
 
@@ -1197,8 +1335,11 @@ void EditWindow::saveConfig()
     // Write new window size back to registry
     getApp()->reg().writeUnsignedEntry("OPTIONS","width",getWidth());
     getApp()->reg().writeUnsignedEntry("OPTIONS","height",getHeight());
-    getApp()->reg().writeIntEntry("OPTIONS","xpos",getX());
-    getApp()->reg().writeIntEntry("OPTIONS","ypos",getY());
+	if (save_win_pos)
+	{
+		getApp()->reg().writeIntEntry("OPTIONS","xpos",getX());
+		getApp()->reg().writeIntEntry("OPTIONS","ypos",getY());
+	}
 
     // Was status line shown
     getApp()->reg().writeIntEntry("OPTIONS","hidestatus",!statusbar->shown());
@@ -1226,15 +1367,28 @@ void EditWindow::saveConfig()
     // Font
     fontspec=editor->getFont()->getFont();
     getApp()->reg().writeStringEntry("OPTIONS","textfont",fontspec.text());
+
+	// Write registry options
+	getApp()->reg().write();
+}
+
+
+// Harvest the zombies
+long EditWindow::onSigHarvest(FXObject*,FXSelector,void*)
+{
+	while(waitpid(-1,NULL,WNOHANG)>0);
+	return 1;
 }
 
 
 // About box
 long EditWindow::onCmdAbout(FXObject*,FXSelector,void*)
 {
-    FXString msg=_("X File Write Version ");
-    msg = msg + VERSION + _(" is a simple text editor.\n\nCopyright (C) 2002-2008 Roland Baudin (roland65@free.fr)");
-    MessageBox about(this,_("About X File Write"),msg.text(),xfwicon,BOX_OK|DECOR_TITLE|DECOR_BORDER);
+	FXString msg;
+	msg.format(_("X File Write Version %s is a simple text editor.\n\n"),VERSION);
+	msg += COPYRIGHT;
+	MessageBox about(this,_("About X File Write"),msg.text(),xfwicon,BOX_OK|DECOR_TITLE|DECOR_BORDER,
+	                 JUSTIFY_CENTER_X|ICON_BEFORE_TEXT|LAYOUT_TOP|LAYOUT_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y);
     about.execute(PLACEMENT_OWNER);
     return 1;
 }
@@ -1614,7 +1768,7 @@ long EditWindow::onCmdLinesNum(FXObject*,FXSelector,void*)
 }
 
 
-// Update toggle lines number
+// Update toggle line numbers
 long EditWindow::onUpdLinesNum(FXObject* sender,FXSelector,void*)
 {
     if(linesnum)
