@@ -222,6 +222,7 @@ public:
     long onUpdateTitle(FXObject*,FXSelector,void*);
     long onFocusIn(FXObject*,FXSelector,void*);
     long onCmdAbout(FXObject*,FXSelector,void*);
+	long onSigHarvest(FXObject*,FXSelector,void*);
 
     // File management
     long onCmdNew(FXObject*,FXSelector,void*);
@@ -270,6 +271,7 @@ public:
         ID_OPEN_TREE,
         ID_OPEN_SELECTED,
         ID_OPEN_RECENT,
+        ID_HARVEST,
         ID_SAVE,
         ID_SAVEAS,
         ID_FONT,
