@@ -8,13 +8,13 @@
 #include <FXPNGIcon.h>
 
 #include "xfedefs.h"
-#include "File.h"
+#include "icons.h"
+#include "xfeutils.h"
 #include "FileList.h"
-#include "FileDialog.h"
 #include "InputDialog.h"
 #include "DirHistBox.h"
 #include "MessageBox.h"
-#include "icons.h"
+#include "FileDialog.h"
 
 #define FILELISTMASK (_ICONLIST_EXTENDEDSELECT|_ICONLIST_SINGLESELECT|_ICONLIST_BROWSESELECT|_ICONLIST_MULTIPLESELECT)
 #define FILESTYLEMASK (_ICONLIST_DETAILED|_ICONLIST_MINI_ICONS|_ICONLIST_BIG_ICONS|_ICONLIST_ROWS|_ICONLIST_COLUMNS|_ICONLIST_AUTOSIZE)
@@ -63,23 +63,20 @@ static const FXchar allfiles[]="All Files (*)";
 
 // File selector object
 FileSelector::FileSelector(FXComposite *p,FXObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h):
-        FXPacker(p,opts,x,y,w,h)
+        FXPacker(p,opts,x,y,w,h,0,0,0,0,0,0)
 {
 	FXAccelTable *table=getShell()->getAccelTable();
     target=tgt;
     message=sel;
     
     // Global container
-    FXVerticalFrame* cont=new FXVerticalFrame(this,LAYOUT_FILL_Y|LAYOUT_FILL_X,0,0,0,0, 0,0,0,0, 1,1);
-    
-    // Container for the action buttons
-    FXHorizontalFrame* buttons=new FXHorizontalFrame(cont,LAYOUT_SIDE_TOP|LAYOUT_FILL_X,0,0,0,0, DEFAULT_SPACING,DEFAULT_SPACING,DEFAULT_SPACING,DEFAULT_SPACING, 0,0);
-    
-    // Line
-   	new FXHorizontalFrame(cont,FRAME_SUNKEN|LAYOUT_SIDE_TOP|LAYOUT_FILL_X,0,0,0,0, 0,0,0,0, 0,0);
+    FXVerticalFrame* cont=new FXVerticalFrame(this,LAYOUT_FILL_Y|LAYOUT_FILL_X|FRAME_NONE,0,0,0,0, 0,0,0,0, 0,0);
 
+    // Container for the action buttons
+    FXHorizontalFrame* buttons=new FXHorizontalFrame(cont,LAYOUT_SIDE_TOP|LAYOUT_FILL_X|FRAME_RAISED,0,0,0,0, 5,5,5,5, 0,0);
+    
     // Container for the path linker
-    FXHorizontalFrame* pathframe=new FXHorizontalFrame(cont,LAYOUT_FILL_X,0,0,0,0, 0,0,0,0, 0,0);
+    FXHorizontalFrame* pathframe=new FXHorizontalFrame(cont,LAYOUT_FILL_X|FRAME_RAISED,0,0,0,0, 0,0,0,0, 0,0);
 
  	// File list
  	FXuint options;
@@ -88,7 +85,8 @@ FileSelector::FileSelector(FXComposite *p,FXObject* tgt,FXSelector sel,FXuint op
 		options=_ICONLIST_MINI_ICONS|_ICONLIST_BROWSESELECT|LAYOUT_FILL_X|LAYOUT_FILL_Y;
 	else
 		options=_ICONLIST_MINI_ICONS|_ICONLIST_BROWSESELECT|LAYOUT_FILL_X|LAYOUT_FILL_Y|SCROLLERS_DONT_TRACK;
-    list=new FileList(this,cont,this,ID_FILELIST,options);
+	FXbool thumbnails=getApp()->reg().readUnsignedEntry("FILEDIALOG","thumbnails",FALSE);
+    list=new FileList(this,cont,this,ID_FILELIST,thumbnails,options);
     
 	// Set list colors and columns size for detailed mode
 	list->setTextColor(getApp()->reg().readColorEntry("SETTINGS","listforecolor",FXRGB(0,0,0)));
@@ -105,10 +103,8 @@ FileSelector::FileSelector(FXComposite *p,FXObject* tgt,FXSelector sel,FXuint op
 	// Set file selector options
 	FXuint listmode=getApp()->reg().readUnsignedEntry("FILEDIALOG","listmode",_ICONLIST_MINI_ICONS);
 	FXbool hiddenfiles=getApp()->reg().readUnsignedEntry("FILEDIALOG","hiddenfiles",FALSE);
-	FXbool thumbnails=getApp()->reg().readUnsignedEntry("FILEDIALOG","thumbnails",FALSE);
 	showHiddenFiles(hiddenfiles);
 	setFileBoxStyle	(listmode);
-	showThumbnails(thumbnails);
 
     // Entry buttons
     FXMatrix* fields=new FXMatrix(cont,3,MATRIX_BY_COLUMNS|LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X);
@@ -125,39 +121,106 @@ FileSelector::FileSelector(FXComposite *p,FXObject* tgt,FXSelector sel,FXuint op
     cancel=new FXButton(fields,_("&Cancel"),NULL,NULL,0,BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_FILL_X,0,0,0,0,20,20);
 
     // Action buttons
-    new FXFrame(buttons,LAYOUT_FIX_WIDTH,0,0,4,1);
-    new FXButton(buttons,_("\tGo back\tMove to previous folder."),dirbackicon,this,ID_DIR_BACK,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
+    FXString key;
+	FXHotKey hotkey;
+	FXButton *btn;
+	FXToggleButton *tglbtn;
+	
+	new FXFrame(buttons,LAYOUT_FIX_WIDTH,0,0,4,1);
+
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","go_back","Ctrl-Backspace");
+    btn=new FXButton(buttons,TAB+_("Go to previous directory")+PARS(key),dirbackicon,this,ID_DIR_BACK,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
+	hotkey=_parseAccel(key);
+	btn->addHotKey(hotkey);
+
 	btnbackhist=new FXArrowButton(buttons,this,ID_DIR_BACK_HIST,LAYOUT_FILL_Y|FRAME_RAISED|FRAME_THICK|ARROW_DOWN|ARROW_TOOLBAR);
-    new FXButton(buttons,_("\tGo forward\tMove to next folder."),dirforwardicon,this,ID_DIR_FORWARD,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
+
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","go_forward","Shift-Backspace");
+    btn=new FXButton(buttons,TAB+_("Go to next directory")+PARS(key),dirforwardicon,this,ID_DIR_FORWARD,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
+	hotkey=_parseAccel(key);
+	btn->addHotKey(hotkey);
+
 	btnforwardhist=new FXArrowButton(buttons,this,ID_DIR_FORWARD_HIST,LAYOUT_FILL_Y|FRAME_RAISED|FRAME_THICK|ARROW_DOWN|ARROW_TOOLBAR);
-    new FXButton(buttons,_("\tGo up one folder\tMove up to higher folder."),dirupicon,this,ID_DIR_UP,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
-    new FXButton(buttons,_("\tGo to home folder\tBack to home folder."),homeicon,this,ID_HOME,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
-    new FXButton(buttons,_("\tGo to working folder\tBack to working folder."),workicon,this,ID_WORK,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
-    new FXButton(buttons,_("\tNew folder\tCreate new folder."),newfoldericon,this,ID_NEWDIR,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
-    new FXButton(buttons,_("\tShow icons\tDisplay folder with big icons."),bigiconsicon,list,FileList::ID_SHOW_BIG_ICONS,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
-    new FXButton(buttons,_("\tShow list\tDisplay folder with small icons."),smalliconsicon,list,FileList::ID_SHOW_MINI_ICONS,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
-    new FXButton(buttons,_("\tShow details\tDisplay detailed folder listing."),detailsicon,list,FileList::ID_SHOW_DETAILS,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
-    new FXToggleButton(buttons,_("\tShow hidden files\tShow hidden files and folders."),_("\tHide hidden Files\tHide hidden files and folders."),showhiddenicon,hidehiddenicon,list,FileList::ID_TOGGLE_HIDDEN,TOGGLEBUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
-    new FXToggleButton(buttons,_("\tShow thumbnails"),_("\tHide thumbnails"),showthumbicon,hidethumbicon,list,FileList::ID_TOGGLE_THUMBNAILS,TOGGLEBUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","go_up","Backspace");
+    btn=new FXButton(buttons,TAB+_("Go to parent directory")+PARS(key),dirupicon,this,ID_DIR_UP,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
+	hotkey=_parseAccel(key);
+	btn->addHotKey(hotkey);
+
+ 	key=getApp()->reg().readStringEntry("KEYBINDINGS","go_home","Ctrl-H");
+    btn=new FXButton(buttons,TAB+_("Go to home directory")+PARS(key),homeicon,this,ID_HOME,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
+	hotkey=_parseAccel(key);
+	btn->addHotKey(hotkey);
+
+ 	key=getApp()->reg().readStringEntry("KEYBINDINGS","go_work","Shift-F2");
+    btn=new FXButton(buttons,TAB+_("Go to working directory")+PARS(key),workicon,this,ID_WORK,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
+	hotkey=_parseAccel(key);
+	btn->addHotKey(hotkey);
+
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","new_folder","F7");
+    btn=new FXButton(buttons,TAB+_("New folder")+PARS(key),newfoldericon,this,ID_NEWDIR,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
+	hotkey=_parseAccel(key);
+	btn->addHotKey(hotkey);
+
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","big_icons","F10");
+    btn=new FXButton(buttons,TAB+_("Big icon list")+PARS(key),bigiconsicon,list,FileList::ID_SHOW_BIG_ICONS,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
+	hotkey=_parseAccel(key);
+	btn->addHotKey(hotkey);
+
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","small_icons","F11");
+    btn=new FXButton(buttons,TAB+_("Small icon list")+PARS(key),smalliconsicon,list,FileList::ID_SHOW_MINI_ICONS,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
+	hotkey=_parseAccel(key);
+	btn->addHotKey(hotkey);
+
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","detailed_file_list","F12");
+    btn=new FXButton(buttons,TAB+_("Detailed file list")+PARS(key),detailsicon,list,FileList::ID_SHOW_DETAILS,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
+	hotkey=_parseAccel(key);
+	btn->addHotKey(hotkey);
+
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","hidden_files","Ctrl-F6");
+    tglbtn=new FXToggleButton(buttons,TAB+_("Show hidden files")+PARS(key),TAB+_("Hide hidden files")+PARS(key),showhiddenicon,hidehiddenicon,list,FileList::ID_TOGGLE_HIDDEN,TOGGLEBUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
+	hotkey=_parseAccel(key);
+	tglbtn->addHotKey(hotkey);
+
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","thumbnails","Ctrl-F7");
+    tglbtn=new FXToggleButton(buttons,TAB+_("Show thumbnails")+PARS(key),TAB+_("Hide thumbnails")+PARS(key),showthumbicon,hidethumbicon,list,FileList::ID_TOGGLE_THUMBNAILS,TOGGLEBUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
+	hotkey=_parseAccel(key);
+	tglbtn->addHotKey(hotkey);
+
+	// Path text
+	pathtext=new TextLabel(pathframe,0,this,0,LAYOUT_FILL_X|LAYOUT_FILL_Y);
+	pathtext->setBackColor(getApp()->getBaseColor());
 
     // Path linker
-    pathlink = new PathLinker(pathframe,list,LAYOUT_FILL_X);
+    pathlink = new PathLinker(pathframe,list,NULL,LAYOUT_FILL_X);
 
     readonly->hide();
   	if(table)
 	{
-    	table->addAccel(MKUINT(KEY_BackSpace,CONTROLMASK),this,FXSEL(SEL_COMMAND,ID_DIR_BACK));
-    	table->addAccel(MKUINT(KEY_BackSpace,SHIFTMASK),this,FXSEL(SEL_COMMAND,ID_DIR_FORWARD));
-    	table->addAccel(MKUINT(KEY_BackSpace,0),this,FXSEL(SEL_COMMAND,ID_DIR_UP));
-    	table->addAccel(MKUINT(KEY_h,CONTROLMASK),this,FXSEL(SEL_COMMAND,ID_HOME));
-    	table->addAccel(MKUINT(KEY_w,CONTROLMASK),this,FXSEL(SEL_COMMAND,ID_WORK));
-    	table->addAccel(MKUINT(KEY_a,CONTROLMASK),list,FXSEL(SEL_COMMAND,FileList::ID_SELECT_ALL));
-    	table->addAccel(MKUINT(KEY_i,CONTROLMASK),list,FXSEL(SEL_COMMAND,FileList::ID_DESELECT_ALL));
+ 		FXString key;
+		FXHotKey hotkey;
+		
+  		key=getApp()->reg().readStringEntry("KEYBINDINGS","select_all","Ctrl-A");
+		hotkey=_parseAccel(key);	
+		table->addAccel(hotkey,list,FXSEL(SEL_COMMAND,FileList::ID_SELECT_ALL));
+  	
+		key=getApp()->reg().readStringEntry("KEYBINDINGS","deselect_all","Ctrl-Z");
+		hotkey=_parseAccel(key);	
+		table->addAccel(hotkey,list,FXSEL(SEL_COMMAND,FileList::ID_DESELECT_ALL));
+	
+  		key=getApp()->reg().readStringEntry("KEYBINDINGS","invert_selection","Ctrl-I");
+		hotkey=_parseAccel(key);	
+		table->addAccel(hotkey,list,FXSEL(SEL_COMMAND,FileList::ID_SELECT_INVERSE));
+	
+		key=getApp()->reg().readStringEntry("KEYBINDINGS","new_file","F2");
+		hotkey=_parseAccel(key);	
+		table->addAccel(hotkey,this,FXSEL(SEL_COMMAND,ID_NEWFILE));
     }
-    setSelectMode(SELECT_FILE_ANY);          // For backward compatibility, this HAS to be the default!
+
+    setSelectMode(SELECT_FILE_ANY);                          // For backward compatibility, this HAS to be the default!
     setPatternList(allfiles);
-    setDirectory(FXSystem::getCurrentDirectory());      // Update file list
+    setDirectory(FXSystem::getCurrentDirectory());           // Update file list
    	pathlink->setPath(FXSystem::getCurrentDirectory());      // Update path linker
+   	pathtext->setText(FXSystem::getCurrentDirectory());      // Update path text
 
     list->setFocus();
     accept->hide();
@@ -168,6 +231,27 @@ FileSelector::FileSelector(FXComposite *p,FXObject* tgt,FXSelector sel,FXuint op
 }
 
 
+// Create X window
+void FileSelector::create()
+{
+	// Display or hide path linker
+	FXbool show_pathlink=getApp()->reg().readUnsignedEntry("SETTINGS","show_pathlinker",TRUE);
+	if (show_pathlink)
+	{
+		pathtext->hide();
+		pathlink->show();
+	}
+	else
+	{
+		pathlink->hide();
+		pathtext->show();
+	}
+	FXPacker::create();
+	
+}
+
+
+
 // Double-clicked item in file list
 long FileSelector::onCmdItemDoubleClicked(FXObject*,FXSelector,void* ptr)
 {
@@ -189,6 +273,7 @@ long FileSelector::onCmdItemDoubleClicked(FXObject*,FXSelector,void* ptr)
 		
         setDirectory(pathname);
 		pathlink->setPath(pathname);
+		pathtext->setText(pathname);
         return 1;
     }
 
@@ -244,6 +329,7 @@ long FileSelector::onCmdItemClicked(FXObject*,FXSelector,void* ptr)
 				}				
 				setDirectory(pathname);
 				pathlink->setPath(pathname);
+				pathtext->setText(pathname);
 				return 1;
 			}
 			else if((single_click==SINGLE_CLICK_DIR_FILE) && list->isItemFile(index) && allow)
@@ -389,6 +475,7 @@ long FileSelector::onCmdAccept(FXObject*,FXSelector,void*)
             // Hop over to that directory
             list->setDirectory(path);
             pathlink->setPath(list->getDirectory());
+            pathtext->setText(list->getDirectory());
             filename->setText(FXString::null);
             return 1;
         }
@@ -455,6 +542,7 @@ long FileSelector::onCmdAccept(FXObject*,FXSelector,void*)
         // Switch as far as we could go
         list->setDirectory(dir);
         pathlink->setPath(list->getDirectory());
+        pathtext->setText(list->getDirectory());
 
         // Put the tail end back for further editing
         if(ISPATHSEP(path[dir.length()]))
@@ -476,6 +564,7 @@ long FileSelector::onCmdDirUp(FXObject*,FXSelector,void*)
 {
     setDirectory(FXPath::upLevel(list->getDirectory()));
    	pathlink->setPath(list->getDirectory());
+   	pathtext->setText(list->getDirectory());
     return 1;
 }
 
@@ -514,6 +603,7 @@ long  FileSelector::onCmdDirBack(FXObject*,FXSelector s,void* p)
 	// Go to to the previous directory
 	list->setDirectory(pathname,FALSE);
    	pathlink->setPath(list->getDirectory());
+   	pathtext->setText(list->getDirectory());
 
     return 1;
 }
@@ -561,6 +651,7 @@ long  FileSelector::onCmdDirForward(FXObject*,FXSelector s,void* p)
 	// Go to to the previous directory
 	list->setDirectory(pathname,FALSE);
    	pathlink->setPath(list->getDirectory());
+   	pathtext->setText(list->getDirectory());
 
     return 1;
 }
@@ -645,6 +736,7 @@ long  FileSelector::onCmdDirBackHist(FXObject *sender,FXSelector sel,void* ptr)
 			pathname=dirs[pos];
 			list->setDirectory(pathname,FALSE);
    			pathlink->setPath(list->getDirectory());
+   			pathtext->setText(list->getDirectory());
 		}
 		delete[]dirs;
 	}
@@ -731,6 +823,7 @@ long  FileSelector::onCmdDirForwardHist(FXObject *sender,FXSelector sel,void* pt
 			pathname=dirs[pos];
 			list->setDirectory(pathname,FALSE);
    			pathlink->setPath(list->getDirectory());
+   			pathtext->setText(list->getDirectory());
 		}
 		delete[]dirs;
 	}
@@ -763,6 +856,7 @@ long FileSelector::onCmdHome(FXObject*,FXSelector,void*)
 {
     setDirectory(FXSystem::getHomeDirectory());
    	pathlink->setPath(list->getDirectory());
+   	pathtext->setText(list->getDirectory());
     return 1;
 }
 
@@ -785,15 +879,18 @@ long FileSelector::onCmdNewDir(FXObject*,FXSelector,void*)
         if(dirname!=dirpath)
         {
 			// Create the new dir according to the current umask
-			int mask;
+			FXint mask;
 			mask=umask(0);
 			umask(mask);
-			// note that the umask value is in decimal (511 means octal 0777)
+			
+			// Note that the umask value is in decimal (511 means octal 0777)
+			errno=0;
 			FXint ret=::mkdir(dirname.text(),511 & ~mask);
+			FXint errcode=errno;
 			if (ret==-1)
 			{
-				if (errno)
-					MessageBox::error(this,BOX_OK_SU,_("Error"),"Can't create folder %s: %s",dirname.text(),strerror(errno));
+				if (errcode)
+					MessageBox::error(this,BOX_OK_SU,_("Error"),"Can't create folder %s: %s",dirname.text(),strerror(errcode));
 				else
 					MessageBox::error(this,BOX_OK_SU,_("Error"),"Can't create folder %s",dirname.text());
     			return 0;
@@ -835,6 +932,7 @@ long FileSelector::onCmdNewFile(FXObject*,FXSelector,void*)
         		return 0;
 			}
 			// Create the new file
+			errno=0;
 			if (!(file=fopen(filename.text(),"w+"))  ||  fclose(file) )
 			{
 				if (errno)
@@ -844,9 +942,10 @@ long FileSelector::onCmdNewFile(FXObject*,FXSelector,void*)
     			return 0;
 			}
 			// Change the file permissions according to the current umask
-			int mask;
+			FXint mask;
 			mask=umask(0);
 			umask(mask);
+			errno=0;
 			if (chmod(filename.text(), 438 & ~mask) != 0)
 			{
 				if (errno)
@@ -871,6 +970,7 @@ long FileSelector::onCmdWork(FXObject*,FXSelector,void*)
 {
     setDirectory(FXSystem::getCurrentDirectory());
    	pathlink->setPath(list->getDirectory());
+	pathtext->setText(list->getDirectory());
     return 1;
 }
 
@@ -940,6 +1040,7 @@ void FileSelector::setDirectory(const FXString& path)
     FXString abspath=FXPath::absolute(path);
     list->setDirectory(abspath);
    	pathlink->setPath(list->getDirectory());
+   	pathtext->setText(list->getDirectory());
 
     if(selectmode!=SELECT_FILE_ANY)
         filename->setText(FXString::null);
@@ -959,6 +1060,7 @@ void FileSelector::setFilename(const FXString& path)
     FXString abspath=FXPath::absolute(path);
     list->setCurrentFile(abspath);
    	pathlink->setPath(FXPath::directory(abspath));
+   	pathtext->setText(FXPath::directory(abspath));
     filename->setText(FXPath::name(abspath));
 }
 
@@ -1105,7 +1207,7 @@ FXString FileSelector::getPattern() const
 // Set current file pattern from the list
 void FileSelector::setCurrentPattern(FXint patno)
 {
-    if(patno<0 || patno>=filefilter->getNumItems())
+	if ((unsigned int)patno >= (unsigned int)filefilter->getNumItems())
         fxerror("%s::setCurrentPattern: index out of range.\n",getClassName());
     filefilter->setCurrentItem(patno);
     list->setPattern(patternFromText(filefilter->getItemText(patno)));
@@ -1122,7 +1224,7 @@ FXint FileSelector::getCurrentPattern() const
 // Change pattern for pattern number patno
 void FileSelector::setPatternText(FXint patno,const FXString& text)
 {
-    if(patno<0 || patno>=filefilter->getNumItems())
+	if ((unsigned int)patno >= (unsigned int)filefilter->getNumItems())
         fxerror("%s::setPatternText: index out of range.\n",getClassName());
     filefilter->setItemText(patno,text);
     if(patno==filefilter->getCurrentItem())
@@ -1133,7 +1235,7 @@ void FileSelector::setPatternText(FXint patno,const FXString& text)
 // Return pattern text of pattern patno
 FXString FileSelector::getPatternText(FXint patno) const
 {
-    if(patno<0 || patno>=filefilter->getNumItems())
+	if ((unsigned int)patno >= (unsigned int)filefilter->getNumItems())
         fxerror("%s::getPatternText: index out of range.\n",getClassName());
     return filefilter->getItemText(patno);
 }
@@ -1283,6 +1385,7 @@ FileSelector::~FileSelector()
     }
     delete list;
     delete pathlink;
+	delete pathtext;
 	delete filename;
 	delete filefilter;
 	delete readonly;
