@@ -35,9 +35,11 @@ protected:
 	FXArrowButton   *btnbackhist;     // Back history
 	FXArrowButton   *btnforwardhist;  // Forward history
     PathLinker*		  pathlink;
+	TextLabel*	      pathtext;
 protected:
     FileSelector()
     {}
+    virtual void create();
     static FXString patternFromText(const FXString& pattern);
     static FXString extensionFromPattern(const FXString& pattern);
 private:
