@@ -132,13 +132,14 @@ protected:
     FXuint       matchmode;             // File wildcard match mode
     FXTime       timestamp;             // Time when last refreshed
 	FXuint       counter;               // Refresh counter
-	FXbool		ignorecase;				// Case sensitivity for file name sorting
 	FXbool		allowrefresh;			// Allow or disallow periodic refresh
 	FXbool		displaythumbnails;		// Display thumbnails
-	FXString	trashlocation;          // Location of the trash directory
+	FXString	trashfileslocation;     // Location of the trash files directory
+	FXString	trashinfolocation;      // Location of the trash info directory
 	FXbool      single;                 // Allow single click navigation for directories
 	FXbool		dirsfirst;				// Sort directories first
 	FXint		deldatesize;
+	FXint		origpathsize;
 	FXWindow	*focuswindow;			// Window used to test focus
 
 public:
@@ -188,6 +189,8 @@ public:
 	long onUpdSortByPerm(FXObject*,FXSelector,void*);
 	long onCmdSortByDeltime(FXObject*,FXSelector,void*);
 	long onUpdSortByDeltime(FXObject*,FXSelector,void*);
+	long onCmdSortByOrigpath(FXObject*,FXSelector,void*);
+	long onUpdSortByOrigpath(FXObject*,FXSelector,void*);
 	long onCmdSortReverse(FXObject*,FXSelector,void*);
 	long onUpdSortReverse(FXObject*,FXSelector,void*);
 	long onCmdSortCase(FXObject*,FXSelector,void*);
@@ -236,6 +239,8 @@ public:
 	static FXint descendingPerm(const IconItem* pa,const IconItem* pb);
 	static FXint ascendingDeltime(const IconItem* pa,const IconItem* pb);
 	static FXint descendingDeltime(const IconItem* pa,const IconItem* pb);
+	static FXint ascendingOrigpath(const IconItem* pa,const IconItem* pb);
+	static FXint descendingOrigpath(const IconItem* pa,const IconItem* pb);
 	static FXint ascendingMix(const IconItem* pa,const IconItem* pb);
 	static FXint descendingMix(const IconItem* pa,const IconItem* pb);
 	static FXint ascendingCaseMix(const IconItem* pa,const IconItem* pb);
@@ -260,9 +265,11 @@ public:
 	static FXint descendingPermMix(const IconItem* pa,const IconItem* pb);
 	static FXint ascendingDeltimeMix(const IconItem* pa,const IconItem* pb);
 	static FXint descendingDeltimeMix(const IconItem* pa,const IconItem* pb);
+	static FXint ascendingOrigpathMix(const IconItem* pa,const IconItem* pb);
+	static FXint descendingOrigpathMix(const IconItem* pa,const IconItem* pb);
 public:
 	enum {
-		// Note : the order of the 9 following sort IDs must be kept
+		// Note : the order of the 10 following sort IDs must be kept
         ID_SORT_BY_NAME=IconList::ID_LAST,
         ID_SORT_BY_SIZE,
         ID_SORT_BY_TYPE,
@@ -271,6 +278,7 @@ public:
         ID_SORT_BY_USER,
         ID_SORT_BY_GROUP,
         ID_SORT_BY_PERM,   
+        ID_SORT_BY_ORIGPATH,
         ID_SORT_BY_DELTIME,
         ID_SORT_BY_DIRNAME,
         ID_SORT_REVERSE,
@@ -295,7 +303,7 @@ public:
 public:
 
     // Construct a file list
-    FileList(FXWindow* focuswin, FXComposite *p,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0);
+    FileList(FXWindow* focuswin, FXComposite *p,FXObject* tgt=NULL,FXSelector sel=0,FXbool showthumbs=FALSE,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0);
 
     // Create server-side resources
     virtual void create();
@@ -336,11 +344,40 @@ public:
     // Return TRUE if item is executable
     FXbool isItemExecutable(FXint index) const;
 
+    // Return TRUE if item is a symbolic link
+    FXbool isItemLink(FXint index) const;
+
+	// Get number of selected items
+	FXint getNumSelectedItems(void) const
+	{
+		FXint num=0;
+		for (int u=0; u<getNumItems(); u++)
+			if (isItemSelected(u))
+				num++;
+		return num;
+	}
+
+
+	// Get number of selected items and index of first selected item
+	FXint getNumSelectedItems(FXint *index) const
+	{
+		FXint num=0, itm=-1;
+		for (int u=0; u<getNumItems(); u++)
+			if (isItemSelected(u))
+			{
+				if (itm== -1)
+					itm= u;
+				num++;
+			}
+		(*index)=itm;
+		return num;
+	}
+
 	// Return number of selected items
-	FXint getNumSelectedItems(void) const;
+	//FXint getNumSelectedItems(void) const;
 
 	// Return number of selected items and index of first selected item  
-	FXint getNumSelectedItems(FXint *index) const;
+	//FXint getNumSelectedItems(FXint *index) const;
 	
 	// Return name of item at index
     FXString getItemFilename(FXint index) const;
@@ -363,18 +400,6 @@ public:
         return matchmode;
     }
 
-	// Return case sensitivity for file name sorting
-	FXbool getIgnoreCase() const
-	{
-		return ignorecase;
-	}
-
-	// Set case sensitivity for file name sorting
-	void setIgnoreCase(const FXbool icase)
-	{
-		ignorecase=icase;
-	}
-
 	// Return directory first state for file name sorting
 	FXbool getDirsFirst() const
 	{
