@@ -22,14 +22,19 @@ protected:
     FilePanel*		current;
     FileList*	  	list;
     FilePanel*		next;
+	DirPanel*		dirpanel;
     PathLinker*		pathlink;
 	FXPacker* 		statusbar;
-    FXLabel*		status;
+    FXLabel*		statuslabel;
+    FXLabel*		filterlabel;
+	FXLabel* 		activeicon;
     FXString 		name;
 	FXbool			ctrlkey;
 	FXbool			selmult;
 	FXString		homelocation;
 	FXString		trashlocation;
+	FXString		trashfileslocation;
+	FXString		trashinfolocation;
 	FXString		startlocation;
 	FXDragCorner    *corner;
 	FXDragType      urilistType;        // Standard uri-list type
@@ -48,23 +53,32 @@ protected:
 	FXbool			paste;
 	FXbool 			ctrlflag;			// Flag to select the right click control menu
 	FXbool			shiftf10;           // Flag indicating that Shift-F10 was pressed 
+	TextLabel*		pathtext;
+	FXbool			isactive;
+	FXbool			stopListRefresh;
+	FXColor			attenclr;
+	
 public:
 
-FilePanel(FXWindow *owner, const FXchar*,FXComposite*, FXuint name_size=200, FXuint size_size=60, FXuint type_size=100, FXuint ext_size=100, 
-	      FXuint modd_size=150, FXuint user_size=50, FXuint grou_size=50, FXuint attr_size=100, FXuint deldate_size=150,
-          FXColor listbackcolor=FXRGB(255,255,255), FXColor listforecolor=FXRGB(0,0,0), FXbool smoothscroll=TRUE,
+FilePanel(FXWindow *owner, const FXchar*,FXComposite*, DirPanel*, FXuint name_size=200, FXuint size_size=60, FXuint type_size=100, FXuint ext_size=100, 
+	      FXuint modd_size=150, FXuint user_size=50, FXuint grou_size=50, FXuint attr_size=100, FXuint deldate_size=150, FXuint origpath_size=200, FXbool showthumbs=FALSE,
+          FXColor listbackcolor=FXRGB(255,255,255), FXColor listforecolor=FXRGB(0,0,0), FXColor attentioncolor=FXRGB(255,0,0), FXbool smoothscroll=TRUE,
  		  FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0);
     FilePanel()
     {}
     virtual void create();
     ~FilePanel();
-    void focus();
-    void unfocus();
+
+    void setActive();
+    void setInactive(FXbool=TRUE);
+
     void Next(FilePanel*);
 	void updateLocation();
 	void showCorner(FXbool show);
-   enum {
+	enum
+	{
         ID_FILELIST=FXVerticalFrame::ID_LAST,
+		ID_STOP_LIST_REFRESH_TIMER,
         ID_DIRECTORY_UP,
         ID_VIEW,
         ID_EDIT,
@@ -77,6 +91,7 @@ FilePanel(FXWindow *owner, const FXchar*,FXComposite*, FXuint name_size=200, FXu
         ID_FILE_SYMLINK,
         ID_FILE_DELETE,
         ID_FILE_TRASH,
+        ID_FILE_RESTORE,
         ID_FILE_ASSOC,
 		ID_POPUP_MENU,
 		ID_XTERM,
@@ -95,6 +110,7 @@ FilePanel(FXWindow *owner, const FXchar*,FXComposite*, FXuint name_size=200, FXu
         ID_OPEN,
         ID_OPEN_WITH,
         ID_FILTER,
+		ID_FILTER_CURRENT,
         ID_STATUS,
         ID_LABEL,
         ID_REFRESH,
@@ -122,7 +138,7 @@ public:
     long onUpdStatus(FXObject*,FXSelector,void*);
     long onCmdItemDoubleClicked(FXObject*,FXSelector,void*);
     long onCmdItemClicked(FXObject*,FXSelector,void*);
-	long onCmdItemFocus(FXObject*,FXSelector, void*);
+	long onCmdFocus(FXObject*,FXSelector, void*);
     long onCmdItemFilter(FXObject*,FXSelector,void*);
     long onCmdCopyCut(FXObject*,FXSelector,void*);
     long onCmdPaste(FXObject*,FXSelector,void*);
@@ -133,6 +149,7 @@ public:
     long onCmdProperties(FXObject*,FXSelector,void*);
     long onCmdFileMan(FXObject*,FXSelector,void*);
     long onCmdFileTrash(FXObject*,FXSelector,void*);
+    long onCmdFileRestore(FXObject*,FXSelector,void*);
 	long onCmdFileDelete(FXObject*,FXSelector,void*);
     long onCmdFileAssoc(FXObject*,FXSelector,void*);
     long onCmdNewDir(FXObject*,FXSelector,void*);
@@ -162,8 +179,10 @@ public:
 	long onUpdSelMult(FXObject*,FXSelector,void*);
 	long onUpdFileDelete(FXObject*,FXSelector,void*);
 	long onUpdFileTrash(FXObject*,FXSelector,void*);
+	long onUpdFileRestore(FXObject*,FXSelector,void*);
 	long onUpdGoTrash(FXObject*,FXSelector,void*);
-	void updatePathLinker();
+	void updatePath();
+	long onCmdStopListRefreshTimer(FXObject*,FXSelector,void*);
 
 #if defined(linux)
     long onCmdMount(FXObject*,FXSelector,void*);
@@ -175,6 +194,19 @@ public:
     long onCmdPkgUninstall(FXObject*,FXSelector,void*);
 #endif
 public:
+
+	// Change path text
+	void setPathText(FXString title)
+	{
+		pathtext->setText(title);
+	}
+	
+	// Toggle FileList refresh
+	void setAllowRefresh(FXbool flag)
+	{
+		list->setAllowRefresh(flag);
+	}
+
 	// Change sort function
 	void setSortFunc(IconListSortFunc func)
 	{
@@ -205,6 +237,12 @@ public:
 		return current;
 	}
 
+	// Return a pointer on the next panel
+	FilePanel* getNext(void) const
+	{
+		return next;
+	}
+	
 	// Set current directory
 	void setDirectory(FXString pathname, FXbool notify=FALSE)
 	{
@@ -301,10 +339,10 @@ public:
 		list->setFocus();
 	}
 
-	// Has focus?
-	FXbool hasFocus(void)
+	// Is panel active?
+	FXbool isActive(void)
 	{
-		return list->hasFocus();
+		return isactive;
 	}
 
 	// Get current item
@@ -317,6 +355,7 @@ public:
 	void setCurrentItem(FXint item)
 	{
 		list->setCurrentItem(item);
+		list->makeItemVisible(item);		
 	}
 
 	// Select item
@@ -357,7 +396,7 @@ public:
 	}
 
 	// Set path of the path linker
-	void setPathlink(FXString pathname)
+	void setPathLink(FXString pathname)
 	{
 		pathlink->setPath(pathname);
 	}
