@@ -17,22 +17,23 @@ class XComApp;
 class ComboBox : public FXComboBox
 {
     FXDECLARE(ComboBox)
+private:
     ComboBox()
     {
         ;
     }
 public:
-    FXTextField *getTextEntry()
-    {
-        return field;
-    }
-    void CursorEnd()
-    {
+	FXTextField *getTextEntry()
+	{
+		return field;
+	}
+	void CursorEnd()
+	{
 		field->onCmdCursorEnd(0,0,0);
         field->setFocus();
-    }
+	}
     ComboBox(FXComposite *p,FXint cols,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=COMBOBOX_NORMAL);
-
+    virtual void create();
 };
 
 class HistInputDialog : public DialogBox
@@ -42,7 +43,7 @@ protected:
     FXVerticalFrame* contents;
     FXHorizontalFrame* buttons;
     FXHorizontalFrame* checkbutton;
-    ComboBox *input;
+    ComboBox*          input;
     FXLabel*           label;
     FXDataTarget*      string_target;
 	FXuint			   browsetype;
@@ -83,7 +84,7 @@ public:
     {
         return input->getItemText(pos);
     }
-    int getHistorySize()
+    FXint getHistorySize()
     {
         return input->getNumItems();
     }
