@@ -11,7 +11,8 @@ enum {
 		BOX_YES_NO_CANCEL        = 0x40000000, // Message box has YES, NO, and CANCEL buttons
 		BOX_QUIT_CANCEL          = 0x50000000, // Message box has QUIT and CANCEL buttons
 		BOX_QUIT_SAVE_CANCEL     = 0x60000000, // Message box has QUIT, SAVE, and CANCEL buttons
-		BOX_OK_SU 			     = 0x70000000  // Message box has OK and SU buttons
+		BOX_YES_NO_ALL_CANCEL 	 = 0x70000000, // Message box has YES, NO, ALL and CANCEL buttons
+		BOX_OK_SU 			     = 0x80000000  // Message box has OK and SU buttons
 };
 enum {
 		BOX_CLICKED_YES      = 1,            // The YES button was clicked
@@ -20,7 +21,8 @@ enum {
 		BOX_CLICKED_CANCEL   = 4,            // The CANCEL button was clicked
 		BOX_CLICKED_QUIT     = 5,            // The QUIT button was clicked
 		BOX_CLICKED_SAVE     = 6,            // The SAVE button was clicked
-		BOX_CLICKED_SU       = 7             // The SU button was clicked
+		BOX_CLICKED_ALL      = 7,            // The ALL button was clicked
+		BOX_CLICKED_SU       = 8             // The SU button was clicked
  };
   
   
@@ -34,7 +36,7 @@ protected:
 private:
     MessageBox(const MessageBox&);
     MessageBox &operator=(const MessageBox&);
-    void initialize(const FXString& text,FXIcon* ic,FXuint whichbuttons);
+    void initialize(const FXString&,FXIcon*,FXuint,FXuint);
 public:
     long onCmdClicked(FXObject*,FXSelector,void*);
     long onCmdCancel(FXObject*,FXSelector,void*);
@@ -47,16 +49,21 @@ public:
         ID_CLICKED_CANCEL,
         ID_CLICKED_QUIT,
         ID_CLICKED_SAVE,
+        ID_CLICKED_ALL,
         ID_CLICKED_SU,
         ID_LAST
     };
 public:
 
     // Construct message box with given caption, icon, and message text
-    MessageBox(FXWindow* owner,const FXString& caption,const FXString& text,FXIcon* ic=NULL,FXuint opts=0,FXint x=0,FXint y=0);
+    MessageBox(FXWindow* owner,const FXString& caption,const FXString& text,FXIcon* ic=NULL,FXuint opts=0,
+	           FXuint textopts=JUSTIFY_LEFT|ICON_BEFORE_TEXT|LAYOUT_TOP|LAYOUT_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y,
+			   FXint x=0,FXint y=0);
 
     // Construct free floating message box with given caption, icon, and message text
-    MessageBox(FXApp* a,const FXString& caption,const FXString& text,FXIcon* ic=NULL,FXuint opts=0,FXint x=0,FXint y=0);
+    MessageBox(FXApp* a,const FXString& caption,const FXString& text,FXIcon* ic=NULL,FXuint opts=0,
+	           FXuint textopts=JUSTIFY_LEFT|ICON_BEFORE_TEXT|LAYOUT_TOP|LAYOUT_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y,
+			   FXint x=0,FXint y=0);
 
     // Show a modal error message.
     // The text message may contain printf-tyle formatting commands.
