@@ -4,6 +4,7 @@
 #include <vector>
 #include "TextLabel.h"
 #include "FileList.h"
+#include "DirPanel.h"
 
 
 class FXAPI PathLinker : public FXHorizontalFrame
@@ -21,7 +22,7 @@ public:
         ID_LAST
     };
 
-    PathLinker(FXComposite* a, FileList* list, FXuint opts=0);
+    PathLinker(FXComposite* a, FileList* flist, DirList* dlist=NULL, FXuint opts=0);
 	PathLinker()
 	{}
     virtual void create();
@@ -42,6 +43,7 @@ protected:
 	FXFont* normalFont;
 	FXFont* highlightFont;
 	FileList* filelist;
+	DirList* dirlist;
 private:
     void updatePath(FXString, FXuint);
     void setText(FXuint, FXString);
