@@ -21,6 +21,9 @@ private:
     FXCheckButton*	or_;
     FXCheckButton*	ow;
     FXCheckButton*	ox;
+    FXCheckButton*	suid;
+    FXCheckButton*	sgid;
+    FXCheckButton*	svtx;
 	FXDataTarget	cmd_radiotarget;
 	FXDataTarget	flt_radiotarget;
 	FXRadioButton*	set;
@@ -34,7 +37,6 @@ private:
     FXCheckButton*	rec;
     FXCheckButton*	own;
 	FXComboBox*		user;
-    FXComboBox*		userbis;
     FXComboBox*		grp;
 public:
     PermFrame(FXComposite *parent,FXObject *target);
@@ -47,16 +49,31 @@ class PropertiesBox : public DialogBox
 private:
     PropertiesBox()
     {}
-	FXLabel		*fileSize;
-	FXLabel		*location;
-	FXLabel		*linkto;
-    FXString 	*files;
-	FXLabel		*name_encoding;
+	FXLabel*	fileSize;
+	FXLabel*	filesSize;
+	FXLabel*	filesSizeDetails;
+	FXLabel*	location;
+	FXLabel*	origlocation;
+	FXLabel*	linkto;
+	FXLabel*	deletiondate;
+    FXLabel*	ext;
+    FXString*	files;
+	FXLabel*	name_encoding;
     FXString 	source;
 	FXString	parentdir;
+	FXString	filename;
     FXString 	oldusr;
     FXString 	oldgrp;
-    int num;
+    FXString	descr_prev;
+    FXString	open_prev;
+    FXString	view_prev;
+    FXString	edit_prev;
+    FXString	bigic_prev;
+    FXString	miniic_prev;
+    FXint 		num;
+	FXString		homelocation;
+	FXString		trashfileslocation;
+	FXString		trashinfolocation;
     FXCheckButton*    ur;
     FXCheckButton*    uw;
     FXCheckButton*    ux;
@@ -66,20 +83,27 @@ private:
     FXCheckButton*    or_;
     FXCheckButton*    ow;
     FXCheckButton*    ox;
-    FXTextField		*input;
-    FXTextField		*username;
-    FXTextField		*grpname;
-    FXTextField		*open;
-    FXTextField		*view;
-    FXTextField		*edit;
-    FXTextField		*ext;
-    FXTextField		*descr;
-    FXTextField		*bigic;
-    FXTextField		*miniic;
-    FXTextField		*test;
+    FXbool			executable;
+#ifdef STARTUP_NOTIFICATION
+    FXCheckButton*	snbutton;
+    FXGroupBox*		sngroup;
+    FXbool			sndisable_prev;
+#endif
+    FXTextField*	input;
+    FXTextField*	username;
+    FXTextField*	grpname;
+    FXTextField*	open;
+    FXTextField*	view;
+    FXTextField*	edit;
+    FXTextField*	descr;
+    FXTextField*	bigic;
+    FXTextField*	miniic;
+	FXbool			isDirectory;
+	FXbool			isMountpoint;
+	FXbool			recsize;
     mode_t			mode;
     mode_t			orig_mode;
-    PermFrame 		*perm;
+    PermFrame*		perm;
 public:
     enum{
         ID_ACCEPT_SINGLE=DialogBox::ID_LAST,
@@ -90,6 +114,7 @@ public:
         ID_ADD,
 		ID_DIRONLY,
 		ID_FILEONLY,
+		ID_SNDISABLE,
 		ID_ALL,
         ID_BIG_ICON,
         ID_MINI_ICON,
@@ -105,6 +130,9 @@ public:
         ID_ROTH,
         ID_WOTH,
         ID_XOTH,
+		ID_SUID,
+		ID_SGID,
+		ID_SVTX,
         ID_LAST
     };
 
@@ -120,8 +148,11 @@ public:
     long onCmdFilter(FXObject*,FXSelector,void*);
     long onCmdBrowseIcon(FXObject*,FXSelector,void*);
     long onCmdBrowse(FXObject*,FXSelector,void*);
-	long onUpdPerm(FXObject*,FXSelector,void*);
+	long onUpdSizeAndPerm(FXObject*,FXSelector,void*);
     long onCmdKeyPress(FXObject*,FXSelector,void*);
+#ifdef STARTUP_NOTIFICATION
+	long onUpdSnDisable(FXObject*,FXSelector,void*);
+#endif
 };
 
 #endif
