@@ -1373,7 +1373,11 @@ FXbool TextLabel::setSelection(FXint pos,FXint len)
 // Extend selection
 FXbool TextLabel::extendSelection(FXint pos)
 {
-    FXDragType types[4];
+	// Don't select text if ncols=0
+	if (columns==0)
+		return TRUE;
+		
+	FXDragType types[4];
 
     // Validate position to start of character
     pos=contents.validate(FXCLAMP(0,pos,contents.length()));
