@@ -8,6 +8,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/stat.h>
+#include <sys/wait.h>
 #include <string.h>
 #include <unistd.h>
 
@@ -17,6 +18,8 @@
 
 #include "xfedefs.h"
 #include "icons.h"
+#include "xfeutils.h"
+#include "startupnotification.h"
 #include "FontDialog.h"
 #include "InputDialog.h"
 #include "FileDialog.h"
@@ -24,6 +27,7 @@
 #include "ViewWindow.h"
 #include "XFileView.h"
 
+FXbool save_win_pos;
 
 // Map
 FXDEFMAP(ViewWindow) ViewWindowMap[]=
@@ -34,6 +38,7 @@ FXDEFMAP(ViewWindow) ViewWindowMap[]=
 	FXMAPFUNC(SEL_UPDATE,0,ViewWindow::onUpdTitle),
 	FXMAPFUNC(SEL_COMMAND,ViewWindow::ID_FIND,ViewWindow::onCmdFind),
 	FXMAPFUNC(SEL_COMMAND,ViewWindow::ID_FIND_AGAIN,ViewWindow::onCmdFind),
+	FXMAPFUNC(SEL_SIGNAL,ViewWindow::ID_HARVEST,ViewWindow::onSigHarvest),
 	FXMAPFUNC(SEL_COMMAND,ViewWindow::ID_QUIT,ViewWindow::onCmdQuit),
 	FXMAPFUNC(SEL_COMMAND,ViewWindow::ID_PRINT,ViewWindow::onCmdPrint),
 	FXMAPFUNC(SEL_COMMAND,ViewWindow::ID_FONT,ViewWindow::onCmdFont),
@@ -53,13 +58,13 @@ FXIMPLEMENT(ViewWindow,FXMainWindow,ViewWindowMap,ARRAYNUMBER(ViewWindowMap))
 ViewWindow::ViewWindow(XFileView* a,const FXString& file):FXMainWindow(a,"XFileView",NULL,NULL,DECOR_ALL),mrufiles(a)
 {
     FXHotKey hotkey;
-    FXButton *btn = NULL;
+	FXString key;
 
     // Add to list of windows
     getApp()->windowlist.append(this);
 	
+	// Icon
 	setIcon(xfvicon);
-	//setMiniIcon(xfvsmallicon);
 	
     // Default font
     font=NULL;
@@ -75,7 +80,7 @@ ViewWindow::ViewWindow(XFileView* a,const FXString& file):FXMainWindow(a,"XFileV
 
     // Tool bar
   	FXToolBarShell* dragshell1=new FXToolBarShell(this,FRAME_RAISED);
-  	toolbar=new FXToolBar(topdock,dragshell1,LAYOUT_DOCK_NEXT|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|FRAME_RAISED);
+  	toolbar=new FXToolBar(topdock,dragshell1,LAYOUT_DOCK_NEXT|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_RAISED);
   	new FXToolBarGrip(toolbar,toolbar,FXToolBar::ID_TOOLBARGRIP,TOOLBARGRIP_DOUBLE);
     int style=BUTTON_TOOLBAR;
 
@@ -108,7 +113,7 @@ ViewWindow::ViewWindow(XFileView* a,const FXString& file):FXMainWindow(a,"XFileV
     new FXMenuTitle(menubar,_("&Help"),NULL,helpmenu);
 
 	// Make contents
-    contents=new FXHorizontalFrame(this,FRAME_SUNKEN|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0, 0,0,0,0, 0,0);
+    contents=new FXHorizontalFrame(this,LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_NONE,0,0,0,0, 0,0,0,0, 0,0);
 
     // Make viewer window
     viewer=new FXText(contents,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_Y);
@@ -128,31 +133,57 @@ ViewWindow::ViewWindow(XFileView* a,const FXString& file):FXMainWindow(a,"XFileV
     new FXLabel(statusbar,_("Line:"),NULL,LAYOUT_RIGHT|LAYOUT_CENTER_Y);
 
     // Toolbar button: File open
-    new FXButton(toolbar,_("\tOpen document (Ctrl-O)\tOpen document file. (Ctrl-O)"),fileopenicon,this,ID_OPEN,style|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","open","Ctrl-O");
+    new FXButton(toolbar,TAB+_("Open")+PARS(key)+TAB+_("Open document.")+PARS(key),fileopenicon,this,ID_OPEN,style|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
+
+    // Toolbar button: File close
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","close","Ctrl-W");
+    new FXButton(toolbar,TAB+_("Close")+PARS(key)+TAB+_("Close document.")+PARS(key),closefileicon,this,ID_CLOSE,style|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
 
     // Toolbar button: Print
-	new FXButton(toolbar,_("\tPrint document (Ctrl-P)\tPrint document file. (Ctrl-P)"),printicon,this,ID_PRINT,style|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
+    new FXSeparator(toolbar,SEPARATOR_GROOVE);
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","print","Ctrl-P");
+	new FXButton(toolbar,TAB+_("Print")+PARS(key)+TAB+_("Print document.")+PARS(key),printicon,this,ID_PRINT,style|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
 
 	// Toolbar button: Quit
-    btn=new FXButton(toolbar,_("\tQuit Xfv (Ctrl-Q)\tQuit Xfv. (Ctrl-Q)"),quiticon,this,ID_QUIT,style|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
-    btn->hide();
-    hotkey=(CONTROLMASK<<16) | KEY_w;
-    btn->addHotKey(hotkey);
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","quit","Ctrl-Q");
+    new FXButton(toolbar,TAB+_("Quit")+PARS(key)+TAB+_("Quit Xfv.")+PARS(key),quiticon,this,ID_QUIT,style|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
 
 	// Toolbar button: Copy
     new FXSeparator(toolbar,SEPARATOR_GROOVE);
-    new FXButton(toolbar,_("\tCopy (Ctrl-C)\tCopy selection to clipboard. (Ctrl-C)"),copy_clpicon,viewer,FXText::ID_COPY_SEL,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","copy","Ctrl-C");
+    new FXButton(toolbar,TAB+_("Copy")+PARS(key)+TAB+_("Copy selection to clipboard.")+PARS(key),copy_clpicon,viewer,FXText::ID_COPY_SEL,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
 
 	// Toolbar buttons: Searching
     new FXSeparator(toolbar,SEPARATOR_GROOVE);
-	//new FXFrame(toolbar,LAYOUT_TOP|LAYOUT_LEFT|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,10,10);
-    new FXButton(toolbar,_("\tFind a string (Ctrl-F)\tFind string in document. (Ctrl-F)"),searchicon,this,ID_FIND,style|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
-    new FXButton(toolbar,_("\tFind again (Ctrl-G)\tFind again. (Ctrl-G)"),find_againicon,this,ID_FIND_AGAIN,style|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","search","Ctrl-F");
+	new FXButton(toolbar,TAB+_("Find")+PARS(key)+TAB+_("Find string in document.")+PARS(key),searchicon,this,ID_FIND,style|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
+	
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","search_next","Ctrl-G");
+	new FXButton(toolbar,TAB+_("Find again")+PARS(key)+TAB+_("Find string again.")+PARS(key),find_againicon,this,ID_FIND_AGAIN,style|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
 
     // File Menu entries
-    new FXMenuCommand(filemenu,_("&Open file...\tCtrl-O\tOpen document file. (Ctrl-O)"),fileopenicon,this,ID_OPEN);
-    new FXMenuCommand(filemenu,_("&Print file...\tCtrl-P\tPrint document file. (Ctrl-P)"),printicon,this,ID_PRINT);
+	FXMenuCommand* mc = NULL;
+	FXString text;
 
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","open","Ctrl-O");
+	text=_("&Open...")+TABS(key)+_("Open document.")+PARS(key);
+    mc=new FXMenuCommand(filemenu,text,fileopenicon,this,ID_OPEN);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
+ 
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","close","Ctrl-W");
+	text=_("&Close")+TABS(key)+_("Close document.")+PARS(key);
+    mc=new FXMenuCommand(filemenu,text,closefileicon,this,ID_CLOSE);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
+
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","print","Ctrl-P");
+	text=_("&Print...")+TABS(key)+_("Print document.")+PARS(key);
+	mc=new FXMenuCommand(filemenu,text,printicon,this,ID_PRINT);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
+ 
     // Recent file menu; this automatically hides if there are no files
     FXMenuSeparator* sep1=new FXMenuSeparator(filemenu);
     sep1->setTarget(&mrufiles);
@@ -167,20 +198,36 @@ ViewWindow::ViewWindow(XFileView* a,const FXString& file):FXMainWindow(a,"XFileV
     sep2->setTarget(&mrufiles);
     sep2->setSelector(FXRecentFiles::ID_ANYFILES);
 
-	new FXMenuCommand(filemenu,_("&Quit\tCtrl-Q\tQuit Xfv. (Ctrl Q)"),quiticon,this,ID_QUIT);
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","quit","Ctrl-Q");
+	text=_("&Quit")+TABS(key)+_("Quit Xfv.")+PARS(key);
+	mc=new FXMenuCommand(filemenu,text,quiticon,this,ID_QUIT);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
 
 	// Edit menu
-	new FXMenuCommand(editmenu,_("&Copy\tCtrl-C\tCopy selection to clipboard. (Ctrl-C)"),copy_clpicon,viewer,FXText::ID_COPY_SEL);
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","copy","Ctrl-C");
+	text=_("&Copy")+TABS(key)+_("Copy selection to clipboard.")+PARS(key);
+	mc=new FXMenuCommand(editmenu,text,copy_clpicon,viewer,FXText::ID_COPY_SEL);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
 
 	// Search Menu entries
-    new FXMenuCommand(searchmenu,_("&Find a string...\tCtrl-F\tFind a string in a document. (Ctrl-F)"),searchicon,this,ID_FIND);
-    new FXMenuCommand(searchmenu,_("Find &again\tCtrl-G\tFind again. (Ctrl-G)"),find_againicon,this,ID_FIND_AGAIN);
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","search","Ctrl-F");
+	text=_("&Find...")+TABS(key)+_("Find a string in a document.")+PARS(key);
+    mc=new FXMenuCommand(searchmenu,text,searchicon,this,ID_FIND);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
 
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","search_again","Ctrl-G");
+	text=_("Find &again")+TABS(key)+_("Find string again.")+PARS(key);
+    mc=new FXMenuCommand(searchmenu,text,find_againicon,this,ID_FIND_AGAIN);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
 
     // Preferences menu
-    new FXMenuCheck(prefsmenu,_("&Toolbar\t\tDisplay or hide toolbar."),toolbar,FXWindow::ID_TOGGLESHOWN);
-    new FXMenuCheck(prefsmenu,_("&Status bar\t\tDisplay or hide status bar."),statusbar,FXWindow::ID_TOGGLESHOWN);
-    new FXMenuCommand(prefsmenu,_("&Font...\t\tChange text font."),fontsicon,this,ID_FONT);
+    new FXMenuCheck(prefsmenu,_("&Toolbar")+TAB2+_("Display or hide toolbar."),toolbar,FXWindow::ID_TOGGLESHOWN);
+    new FXMenuCheck(prefsmenu,_("&Status bar")+TAB2+_("Display or hide status bar."),statusbar,FXWindow::ID_TOGGLESHOWN);
+    new FXMenuCommand(prefsmenu,_("&Font...")+TAB2+_("Change text font."),fontsicon,this,ID_FONT);
 
     // Window menu
     new FXMenuRadio(windowmenu,FXString::null,this,ID_WINDOW_1);
@@ -235,7 +282,11 @@ ViewWindow::ViewWindow(XFileView* a,const FXString& file):FXMainWindow(a,"XFileV
     new FXMenuRadio(windowmenu,FXString::null,this,ID_WINDOW_50);
 
     // Help Menu entries
-    new FXMenuCommand(helpmenu,_("&About X File View\tF1\tAbout X File View. (F1)"),NULL,this,ID_ABOUT,0);
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","help","F1");
+	text=_("&About X File View")+TABS(key)+_("About X File View.")+PARS(key);
+    mc=new FXMenuCommand(helpmenu,text,NULL,this,ID_ABOUT,0);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
 
     // Make a tool tip
     new FXToolTip(getApp(),0);
@@ -259,6 +310,9 @@ ViewWindow::ViewWindow(XFileView* a,const FXString& file):FXMainWindow(a,"XFileV
 	hh=0;
 	xx=0;
 	yy=0;
+	
+	// Initialize search string
+	searchstr="";
 }
 
 
@@ -278,6 +332,10 @@ void ViewWindow::create()
 		font->create();
         viewer->setFont(font);
 	}
+
+    // Retain window position flag
+	save_win_pos=reg_xfe->readUnsignedEntry("SETTINGS","save_win_pos",FALSE);
+
 	delete reg_xfe;
 
 	
@@ -286,8 +344,6 @@ void ViewWindow::create()
 	{
 		ww=getApp()->reg().readUnsignedEntry("OPTIONS","width",DEFAULT_WINDOW_WIDTH);
 		hh=getApp()->reg().readUnsignedEntry("OPTIONS","height",DEFAULT_WINDOW_HEIGHT);
-		xx=getApp()->reg().readIntEntry("OPTIONS","xpos",DEFAULT_WINDOW_XPOS);
-		yy=getApp()->reg().readIntEntry("OPTIONS","ypos",DEFAULT_WINDOW_YPOS);
 	}
 
 	// Get toolbar status
@@ -298,6 +354,18 @@ void ViewWindow::create()
     if(getApp()->reg().readUnsignedEntry("OPTIONS","showstatusbar",TRUE)==FALSE)
         statusbar->hide();
    
+	// Get position and position window    
+    if (save_win_pos && fromreg)
+	{
+		FXint xpos=getApp()->reg().readIntEntry("OPTIONS","xpos",DEFAULT_WINDOW_XPOS);
+		FXint ypos=getApp()->reg().readIntEntry("OPTIONS","ypos",DEFAULT_WINDOW_YPOS);
+		position(xpos,ypos,ww,hh);
+	}
+	else
+	{
+    	position(getX(),getY(),ww,hh);
+	}
+
     FXMainWindow::create();
     filemenu->create();
 	editmenu->create();
@@ -309,8 +377,9 @@ void ViewWindow::create()
     viewer->setFocus();
     viewer->handle(this,FXSEL(SEL_COMMAND,FXText::ID_TOGGLE_EDITABLE),NULL);
 
-	// Reposition window
-	position(xx,yy,ww,hh);
+#ifdef STARTUP_NOTIFICATION
+	startup_completed();
+#endif
 }
 
 
@@ -338,9 +407,11 @@ ViewWindow::~ViewWindow()
 // About box
 long ViewWindow::onCmdAbout(FXObject*,FXSelector,void*)
 {
-    FXString msg=_("X File View Version ");
-	msg = msg + VERSION + _(" is a simple text viewer.\n\nCopyright (C) 2002-2008 Roland Baudin (roland65@free.fr)");
-    MessageBox about(this,_("About X File View"),msg.text(),xfvicon,BOX_OK|DECOR_TITLE|DECOR_BORDER);
+	FXString msg;
+	msg.format(_("X File View Version %s is a simple text viewer.\n\n"),VERSION);
+	msg += COPYRIGHT;
+    MessageBox about(this,_("About X File View"),msg.text(),xfvicon,BOX_OK|DECOR_TITLE|DECOR_BORDER,
+	                 JUSTIFY_CENTER_X|ICON_BEFORE_TEXT|LAYOUT_TOP|LAYOUT_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y);
     about.execute(PLACEMENT_OWNER);
 
     return 1;
@@ -540,8 +611,12 @@ void ViewWindow::saveConfig()
 	// Write new window size and position back to registry
     getApp()->reg().writeUnsignedEntry("OPTIONS","width",(FXuint)getWidth());
     getApp()->reg().writeUnsignedEntry("OPTIONS","height",(FXuint)getHeight());
-    getApp()->reg().writeIntEntry("OPTIONS","xpos",getX());
-    getApp()->reg().writeIntEntry("OPTIONS","ypos",getY());
+
+	if (save_win_pos)
+	{
+		getApp()->reg().writeIntEntry("OPTIONS","xpos",getX());
+		getApp()->reg().writeIntEntry("OPTIONS","ypos",getY());
+	}
 	
     // Toolbar status
     if(toolbar->shown())
@@ -559,15 +634,23 @@ void ViewWindow::saveConfig()
     fontspec=viewer->getFont()->getFont();
     getApp()->reg().writeStringEntry("OPTIONS","textfont",fontspec.text());
 
-	// Write registry OPTIONS
+	// Write registry options
 	getApp()->reg().write();
 }
 
 
+// Harvest the zombies
+long ViewWindow::onSigHarvest(FXObject*,FXSelector,void*)
+{
+	while(waitpid(-1,NULL,WNOHANG)>0);
+	return 1;
+}
+
+
 // Quit
 long ViewWindow::onCmdQuit(FXObject* sender,FXSelector sel,void* ptr)
 {
-	// Save OPTIONS 
+	// Save options
     saveConfig();
 
     // Quit
@@ -603,13 +686,11 @@ long ViewWindow::onUpdMenu(FXObject* sender,FXSelector,void*)
 // Open find dialog
 long ViewWindow::onCmdFind(FXObject* obj,FXSelector sel,void* ptr)
 {
-    int loc;
-
-    static FXString str="";
+    FXint loc;
 
     if(FXSELTYPE(sel)==SEL_UPDATE)
     {
-        if(str=="")
+        if(searchstr=="")
             ((FXWindow*)obj)->disable();
         else
             ((FXWindow*)obj)->enable();
@@ -628,22 +709,22 @@ long ViewWindow::onCmdFind(FXObject* obj,FXSelector sel,void* ptr)
 			new FXButton(buttons,_("&Cancel"),NULL,finddialog,DialogBox::ID_CANCEL,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT,0,0,0,0,20,20);
 			new FXButton(buttons,_("&Find"),NULL,finddialog,DialogBox::ID_ACCEPT,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT);
 		}
-		findtext->setText(str);
-		findtext->setSelection(0,str.length());
+		findtext->setText(searchstr);
+		findtext->setSelection(0,searchstr.length());
 		findtext->setFocus();
 
         if(!finddialog->execute())
             return 0;
-        str=findtext->getText();
+        searchstr=findtext->getText();
     }
-    loc=find(str,viewer->getCursorPos());
+    loc=find(searchstr,viewer->getCursorPos());
     if(loc==-1)
-        MessageBox::information(this,BOX_OK,_("Not Found"),_("String '%s' not found"),str.text());
+        MessageBox::information(this,BOX_OK,_("Not Found"),_("String '%s' not found"),searchstr.text());
     else
     {
-        viewer->setCursorPos(loc+str.length());
+        viewer->setCursorPos(loc+searchstr.length());
         viewer->setTopLine(loc);
-        viewer->setSelection(loc,str.length());
+        viewer->setSelection(loc,searchstr.length());
         viewer->setFocus();
     }
     return 1;
@@ -655,7 +736,7 @@ FXint ViewWindow::find(const FXString str,FXint pos)
 {
     FXString buf=viewer->getText();
     const char* text=buf.text();
-    char *p = strstr(&text[pos],str.text());
+    const char *p = strstr(&text[pos],str.text());
     if(!p)
         return -1;
     return (p-text);
@@ -665,7 +746,7 @@ FXint ViewWindow::find(const FXString str,FXint pos)
 // Close window
 FXbool ViewWindow::close(FXbool notify)
 {
-	// Save OPTIONS 
+	// Save options 
     saveConfig();
 
     return FXMainWindow::close(notify);
