@@ -12,6 +12,7 @@
 #include <dirent.h>
 
 #include "xfedefs.h"
+#include "xfeutils.h"
 #include "FileDict.h"
 #include "FilePanel.h"
 #include "InputDialog.h"
@@ -34,16 +35,16 @@ class XFileExplorer : public FXMainWindow
 protected:
 	enum
 	{
+		TREE_PANEL,
 		ONE_PANEL,
 		TWO_PANELS,
-		TREE_PANEL,
 		TREE_TWO_PANELS,
 		FILEPANEL_FOCUS,
 		DIRPANEL_FOCUS,
 	};	
 	int panel_view;
     int RunHistSize;
-    char RunHistory[RUN_HIST_SIZE][100];
+    char RunHistory[RUN_HIST_SIZE][MAX_COMMAND_SIZE];
 	FXMenuBar		*menubar;
     FXMenuPane		*toolsmenu;
     FXMenuPane		*filemenu;
@@ -68,20 +69,21 @@ protected:
     FXToolBar		*toolstoolbar;
     FXToolBar		*paneltoolbar;
     FXToolBar		*locationbar;
-	FXComboBox		*address;
+	ComboBox		*address;
     FXStatusBar		*status;
     DirPanel		*dirpanel;
     FilePanel		*lpanel;
     FilePanel		*rpanel;
 	FXString		homelocation;
-	FXString		trashlocation;
+	FXString		trashfileslocation;
+	FXString		trashinfolocation;
 	FXString		startlocation;
 	FXuint			liststyle;
 	FXColor			listbackcolor;
 	FXColor			listforecolor;
 	FXColor			highlightcolor;
-    FXButton		*btndel;
-    FXButton		*btntrash;
+	FXColor			pbarcolor;
+	FXColor			attentioncolor;
 	FXArrowButton   *btnbackhist;
 	FXArrowButton   *btnforwardhist;
 	HistInputDialog*   rundialog;
@@ -97,12 +99,15 @@ protected:
 	FXdouble        treepanel_tree_pct;
 	FXdouble        treetwopanels_tree_pct;
 	FXdouble        treetwopanels_lpanel_pct;
+	FXString        prevdir;
+	FXint			prev_width;
 public:
 	enum{
         ID_ABOUT=FXMainWindow::ID_LAST,
 		ID_HELP,
 		ID_REFRESH,
-		ID_EMPTY_TRASHCAN,
+		ID_EMPTY_TRASH,
+		ID_TRASH_SIZE,
         ID_XTERM,
 		ID_DIR_UP,
 		ID_DIR_BACK,
@@ -119,6 +124,7 @@ public:
         ID_FILE_SYMLINK,
         ID_FILE_DELETE,
         ID_FILE_TRASH,
+        ID_FILE_RESTORE,
         ID_FILE_ASSOC,
 		ID_CLEAR_LOCATION,
 		ID_GOTO_LOCATION,
@@ -137,6 +143,7 @@ public:
         ID_NEW_WIN,
         ID_BOOKMARK,
         ID_ADD_BOOKMARK,
+        ID_HARVEST,
         ID_QUIT,
 		ID_LAST
     };
@@ -145,6 +152,7 @@ public:
     virtual void create();
     ~XFileExplorer();
     void saveConfig();
+	long onSigHarvest(FXObject*,FXSelector,void*);
     long onQuit(FXObject*,FXSelector,void*);
 	long onKeyPress(FXObject*,FXSelector,void*);
 	long onCmdHelp(FXObject*,FXSelector,void*);
@@ -157,7 +165,8 @@ public:
     long onCmdRun(FXObject*,FXSelector,void*);
 	long onCmdSu(FXObject*,FXSelector,void*);
     long onCmdXTerm(FXObject*,FXSelector,void*);
-	long onCmdEmptyTrashcan(FXObject*,FXSelector,void*);
+	long onCmdEmptyTrash(FXObject*,FXSelector,void*);
+	long onCmdTrashSize(FXObject*,FXSelector,void*);
 	long onCmdShowPanels(FXObject*,FXSelector,void*);
     long onCmdRestart(FXObject*,FXSelector,void*);
 	long onCmdNewWindow(FXObject*,FXSelector,void*);
@@ -167,11 +176,14 @@ public:
     long onUpdToggleStatus(FXObject*,FXSelector,void*);
 	long onUpdShowPanels(FXObject*,FXSelector,void*);
 	long onUpdFileLocation(FXObject*,FXSelector,void*);
-	long onUpdEmptyTrashcan(FXObject*,FXSelector,void*);
-	long onCmdFileDelete(FXObject*,FXSelector sel,void*);
-	long onCmdFileTrash(FXObject*,FXSelector sel,void*);
+	long onUpdEmptyTrash(FXObject*,FXSelector,void*);
+	long onUpdTrashSize(FXObject*,FXSelector,void*);
+	long onCmdFileDelete(FXObject*,FXSelector,void*);
+	long onCmdFileTrash(FXObject*,FXSelector,void*);
+	long onCmdFileRestore(FXObject*,FXSelector,void*);
 	long onUpdFileDelete(FXObject*,FXSelector,void*);
 	long onUpdFileTrash(FXObject*,FXSelector,void*);
+	long onUpdFileRestore(FXObject*,FXSelector,void*);
 	long onCmdDirUp(FXObject*,FXSelector,void*);
 	long onCmdDirBack(FXObject*,FXSelector,void*);
 	long onUpdDirBack(FXObject*,FXSelector,void*);
@@ -196,6 +208,7 @@ public:
 	long onUpdSynchronizePanels(FXObject*,FXSelector,void*);
 	long onCmdSwitchPanels(FXObject*,FXSelector,void*);
 	long onUpdSwitchPanels(FXObject*,FXSelector,void*);
+	long onUpdSu(FXObject*,FXSelector,void*);
 public:
     // Get associations
     FileDict *getAssociations()
@@ -207,7 +220,7 @@ public:
 	void setDirectory(FXString pathname)
 	{
 		lpanel->getCurrent()->setDirectory(pathname,FALSE);
-		lpanel->getCurrent()->updatePathLinker();
+		lpanel->getCurrent()->updatePath();
 	}
 
 	// Change default cursor for file and dir panels
@@ -231,6 +244,12 @@ public:
 		return lpanel->getCurrent();	
 	}
 
+	// Return a pointer on the next file panel
+	FilePanel* getNextPanel(void)
+	{
+		return lpanel->getNext();	
+	}
+
 	// Return the address box (location bar)
 	FXComboBox* getAddressBox(void)
 	{
