@@ -7,6 +7,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/stat.h>
+#include <sys/wait.h>
 #include <string.h>
 #include <unistd.h>
 #include <signal.h>
@@ -20,11 +21,13 @@
 
 #include "xfedefs.h"
 #include "icons.h"
+#include "xfeutils.h"
+#include "startupnotification.h"
 #include "FileDialog.h"
 #include "InputDialog.h"
 #include "DirHistBox.h"
 #include "MessageBox.h"
-#include "File.h"
+#include "FileList.h"
 #include "XFileImage.h"
 
 // Add FOX hacks
@@ -38,6 +41,7 @@ FXuint single_click;
 FXbool file_tooltips;
 FXbool relative_resize;
 FXbool show_pathlink;
+FXbool save_win_pos;
 
 
 // Hand cursor replacement
@@ -46,30 +50,30 @@ FXbool show_pathlink;
 #define hand_x_hot 6
 #define hand_y_hot 1
  static const FXuchar hand_bits[] = {
-   0x00, 0x00, 0x00, 0x00, 0x60, 0x00, 0x00, 0x00, 0x90, 0x00, 0x00, 0x00,
-   0x90, 0x00, 0x00, 0x00, 0x90, 0x00, 0x00, 0x00, 0x90, 0x07, 0x00, 0x00,
-   0x97, 0x1a, 0x00, 0x00, 0x99, 0x2a, 0x00, 0x00, 0x11, 0x28, 0x00, 0x00,
-   0x12, 0x20, 0x00, 0x00, 0x02, 0x20, 0x00, 0x00, 0x02, 0x20, 0x00, 0x00,
-   0x04, 0x20, 0x00, 0x00, 0x04, 0x10, 0x00, 0x00, 0xf8, 0x0f, 0x00, 0x00,
-   0xf0, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
+	0x00, 0x00, 0x00, 0x00, 0x60, 0x00, 0x00, 0x00, 0x90, 0x00, 0x00, 0x00,
+	0x90, 0x00, 0x00, 0x00, 0x90, 0x00, 0x00, 0x00, 0x90, 0x07, 0x00, 0x00,
+	0x97, 0x1a, 0x00, 0x00, 0x99, 0x2a, 0x00, 0x00, 0x11, 0x28, 0x00, 0x00,
+	0x12, 0x20, 0x00, 0x00, 0x02, 0x20, 0x00, 0x00, 0x02, 0x20, 0x00, 0x00,
+	0x04, 0x20, 0x00, 0x00, 0x04, 0x10, 0x00, 0x00, 0xf8, 0x0f, 0x00, 0x00,
+	0xf0, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
  
  static const FXuchar hand_mask_bits[] = {
-   0x00, 0x00, 0x00, 0x00, 0x60, 0x00, 0x00, 0x00, 0xf0, 0x00, 0x00, 0x00,
-   0xf0, 0x00, 0x00, 0x00, 0xf0, 0x00, 0x00, 0x00, 0xf0, 0x07, 0x00, 0x00,
-   0xf7, 0x1f, 0x00, 0x00, 0xff, 0x3f, 0x00, 0x00, 0xff, 0x3f, 0x00, 0x00,
-   0xfe, 0x3f, 0x00, 0x00, 0xfe, 0x3f, 0x00, 0x00, 0xfe, 0x3f, 0x00, 0x00,
-   0xfc, 0x3f, 0x00, 0x00, 0xfc, 0x1f, 0x00, 0x00, 0xf8, 0x0f, 0x00, 0x00,
-   0xf0, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
+	0x00, 0x00, 0x00, 0x00, 0x60, 0x00, 0x00, 0x00, 0xf0, 0x00, 0x00, 0x00,
+	0xf0, 0x00, 0x00, 0x00, 0xf0, 0x00, 0x00, 0x00, 0xf0, 0x07, 0x00, 0x00,
+	0xf7, 0x1f, 0x00, 0x00, 0xff, 0x3f, 0x00, 0x00, 0xff, 0x3f, 0x00, 0x00,
+	0xfe, 0x3f, 0x00, 0x00, 0xfe, 0x3f, 0x00, 0x00, 0xfe, 0x3f, 0x00, 0x00,
+	0xfc, 0x3f, 0x00, 0x00, 0xfc, 0x1f, 0x00, 0x00, 0xf8, 0x0f, 0x00, 0x00,
+	0xf0, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
    
 
 // Predefined zoom factors 
@@ -102,6 +106,10 @@ const FXchar *patterns[]=
 	NULL
 };
 
+
+const FXString imgpatterns="*.gif,*.bmp,*.xpm,*.pcx,*.ico,*.rgb,*.xbm,*.tga,*.ppm,*.png,*.jpg,*.jpeg,*.tif,*.tiff";
+
+
 // Helper function to draw a toolbar separator
 void toolbarSeparator(FXToolBar* tb)
 {
@@ -118,6 +126,7 @@ FXDEFMAP(XFileImage) XFileImageMap[]=
 	FXMAPFUNC(SEL_COMMAND,XFileImage::ID_ABOUT,XFileImage::onCmdAbout),
 	FXMAPFUNC(SEL_COMMAND,XFileImage::ID_OPEN,XFileImage::onCmdOpen),
 	FXMAPFUNC(SEL_UPDATE,XFileImage::ID_TITLE,XFileImage::onUpdTitle),
+	FXMAPFUNC(SEL_SIGNAL,XFileImage::ID_HARVEST,XFileImage::onSigHarvest),
 	FXMAPFUNC(SEL_COMMAND,XFileImage::ID_QUIT,XFileImage::onCmdQuit),
 	FXMAPFUNC(SEL_SIGNAL,XFileImage::ID_QUIT,XFileImage::onCmdQuit),
 	FXMAPFUNC(SEL_CLOSE,XFileImage::ID_TITLE,XFileImage::onCmdQuit),
@@ -164,7 +173,9 @@ FXDEFMAP(XFileImage) XFileImageMap[]=
 	FXMAPFUNC(SEL_UPDATE,XFileImage::ID_TOGGLE_HIDDEN,XFileImage::onUpdToggleHidden),
 	FXMAPFUNC(SEL_UPDATE,XFileImage::ID_TOGGLE_THUMBNAILS,XFileImage::onUpdToggleThumbnails),
 	FXMAPFUNC(SEL_COMMAND,XFileImage::ID_TOGGLE_FIT_WIN,XFileImage::onCmdToggleFitWin),
+	FXMAPFUNC(SEL_COMMAND,XFileImage::ID_TOGGLE_FILTER_IMAGES,XFileImage::onCmdToggleFilterImages),
 	FXMAPFUNC(SEL_UPDATE,XFileImage::ID_TOGGLE_FIT_WIN,XFileImage::onUpdToggleFitWin),
+	FXMAPFUNC(SEL_UPDATE,XFileImage::ID_TOGGLE_FILTER_IMAGES,XFileImage::onUpdToggleFilterImages),
 	FXMAPFUNC(SEL_COMMAND,XFileImage::ID_HOME,XFileImage::onCmdHome),
 	FXMAPFUNC(SEL_COMMAND,XFileImage::ID_WORK,XFileImage::onCmdWork),
 	FXMAPFUNC(SEL_RIGHTBUTTONRELEASE,XFileImage::ID_FILELIST,XFileImage::onCmdPopupMenu),
@@ -176,12 +187,13 @@ FXIMPLEMENT(XFileImage,FXMainWindow,XFileImageMap,ARRAYNUMBER(XFileImageMap))
 
 
 // Make some windows
-XFileImage::XFileImage(FXApp* a):FXMainWindow(a,"Xfi ",NULL,NULL,DECOR_ALL)
+XFileImage::XFileImage(FXApp* a, FXbool smoothscroll):FXMainWindow(a,"Xfi ",NULL,NULL,DECOR_ALL)
 {
     setIcon(xfiicon);
 
-    FXHotKey hotkey;
-    FXButton *btn = NULL;
+    FXButton* btn=NULL;
+	FXHotKey hotkey;
+	FXString key;
 
     setTarget(this);
     setSelector(ID_TITLE);
@@ -197,7 +209,7 @@ XFileImage::XFileImage(FXApp* a):FXMainWindow(a,"Xfi ",NULL,NULL,DECOR_ALL)
 
     // Tool bar
   	FXToolBarShell* dragshell1=new FXToolBarShell(this,FRAME_RAISED);
-  	toolbar=new FXToolBar(topdock,dragshell1,LAYOUT_DOCK_NEXT|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|FRAME_RAISED);
+  	toolbar=new FXToolBar(topdock,dragshell1,LAYOUT_DOCK_NEXT|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_RAISED);
   	new FXToolBarGrip(toolbar,toolbar,FXToolBar::ID_TOOLBARGRIP,TOOLBARGRIP_DOUBLE);
 
     // File menu
@@ -225,17 +237,16 @@ XFileImage::XFileImage(FXApp* a):FXMainWindow(a,"Xfi ",NULL,NULL,DECOR_ALL)
     splitter=new FXSplitter(splitterbox,LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y|SPLITTER_TRACKING|SPLITTER_VERTICAL|SPLITTER_REVERSED);
 
     // Make image widget
-    imageview=new FXImageView(splitter,NULL,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_Y);
+    imageview=new FXImageView(splitter,NULL,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_SUNKEN);
 
     // Frame box for file list and status bar
-    filebox=new FXHorizontalFrame(splitter,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0, 0,0,0,0);
-    FXVerticalFrame* fileframe=new FXVerticalFrame(filebox,FRAME_SUNKEN|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0, 0,0,0,0, 0,0);
+    filebox=new FXVerticalFrame(splitter,LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_NONE,0,0,0,0, 0,0,0,0, 0,0);
 
     // Container for the action buttons
-    FXHorizontalFrame* buttons=new FXHorizontalFrame(fileframe,LAYOUT_SIDE_TOP|LAYOUT_FILL_X,0,0,0,0, DEFAULT_SPACING,DEFAULT_SPACING,DEFAULT_SPACING,DEFAULT_SPACING, 0,0);
+    FXHorizontalFrame* buttons=new FXHorizontalFrame(filebox,LAYOUT_SIDE_TOP|LAYOUT_FILL_X|FRAME_RAISED,0,0,0,0, 5,5,5,5, 0,0);
 
     // Container for the path linker
-    FXHorizontalFrame* pathframe=new FXHorizontalFrame(fileframe,FRAME_SUNKEN|LAYOUT_FILL_X,0,0,0,0, 0,0,0,0, 0,0);
+    FXHorizontalFrame* pathframe=new FXHorizontalFrame(filebox,LAYOUT_FILL_X|FRAME_RAISED,0,0,0,0, 0,0,0,0, 0,0);
 
 	// File list
 	FXuint options;
@@ -243,7 +254,8 @@ XFileImage::XFileImage(FXApp* a):FXMainWindow(a,"Xfi ",NULL,NULL,DECOR_ALL)
 		options=LAYOUT_FILL_X|LAYOUT_FILL_Y|_ICONLIST_MINI_ICONS|_ICONLIST_BROWSESELECT;
 	else
 		options=LAYOUT_FILL_X|LAYOUT_FILL_Y|_ICONLIST_MINI_ICONS|_ICONLIST_BROWSESELECT|SCROLLERS_DONT_TRACK;
-    filelist=new FileList(this,fileframe,this,ID_FILELIST,options);
+    thumbnails=getApp()->reg().readUnsignedEntry("OPTIONS","thumbnails",0);
+	filelist=new FileList(this,filebox,this,ID_FILELIST,thumbnails,options);
 	filelist->setTextColor(listforecolor);
 	filelist->setBackColor(listbackcolor);
 	filelist->setHeaderSize(0,getApp()->reg().readUnsignedEntry("OPTIONS","name_size",200));
@@ -257,22 +269,26 @@ XFileImage::XFileImage(FXApp* a):FXMainWindow(a,"Xfi ",NULL,NULL,DECOR_ALL)
 
     // Action buttons
     new FXFrame(buttons,LAYOUT_FIX_WIDTH,0,0,4,1);
-    new FXButton(buttons,_("\tGo back\tMove to previous folder."),dirbackicon,this,ID_DIR_BACK,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
+    new FXButton(buttons,TAB+_("Go back")+TAB+_("Move to previous folder."),dirbackicon,this,ID_DIR_BACK,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
 	btnbackhist=new FXArrowButton(buttons,this,ID_DIR_BACK_HIST,LAYOUT_FILL_Y|FRAME_RAISED|FRAME_THICK|ARROW_DOWN|ARROW_TOOLBAR);
-    new FXButton(buttons,_("\tGo forward\tMove to next folder."),dirforwardicon,this,ID_DIR_FORWARD,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
+    new FXButton(buttons,TAB+_("Go forward")+TAB+_("Move to next folder."),dirforwardicon,this,ID_DIR_FORWARD,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
 	btnforwardhist=new FXArrowButton(buttons,this,ID_DIR_FORWARD_HIST,LAYOUT_FILL_Y|FRAME_RAISED|FRAME_THICK|ARROW_DOWN|ARROW_TOOLBAR);
-    new FXButton(buttons,_("\tGo up one folder\tMove up to higher folder."),dirupicon,this,ID_DIR_UP,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
-    new FXButton(buttons,_("\tGo to home folder\tBack to home folder."),homeicon,this,ID_HOME,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
-    new FXButton(buttons,_("\tGo to working folder\tBack to working folder."),workicon,this,ID_WORK,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
-    new FXButton(buttons,_("\tShow icons\tDisplay folder with big icons."),bigiconsicon,this,ID_SHOW_BIG,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
-    new FXButton(buttons,_("\tShow list\tDisplay folder with small icons."),smalliconsicon,this,ID_SHOW_MINI,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
-    new FXButton(buttons,_("\tShow details\tDisplay detailed folder listing."),detailsicon,this,ID_SHOW_DETAILS,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
+    new FXButton(buttons,TAB+_("Go up one folder")+TAB+_("Move up to higher folder."),dirupicon,this,ID_DIR_UP,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
+    new FXButton(buttons,TAB+_("Go to home folder")+TAB+_("Back to home folder."),homeicon,this,ID_HOME,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
+    new FXButton(buttons,TAB+_("Go to working folder")+TAB+_("Back to working folder."),workicon,this,ID_WORK,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
+    new FXButton(buttons,TAB+_("Show icons")+TAB+_("Display folder with big icons."),bigiconsicon,this,ID_SHOW_BIG,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
+    new FXButton(buttons,TAB+_("Show list")+TAB+_("Display folder with small icons."),smalliconsicon,this,ID_SHOW_MINI,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
+    new FXButton(buttons,TAB+_("Show details")+TAB+_("Display detailed folder listing."),detailsicon,this,ID_SHOW_DETAILS,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
+
+	// Panel title
+	pathtext=new TextLabel(pathframe,0,this,0,LAYOUT_FILL_X|LAYOUT_FILL_Y);
+	pathtext->setBackColor(getApp()->getBaseColor());
 
     // Path linker
-    pathlink = new PathLinker(pathframe,filelist,LAYOUT_FILL_X);
+    pathlink = new PathLinker(pathframe,filelist,NULL,LAYOUT_FILL_X);
 
 	// Status bar
-	statusbar=new FXHorizontalFrame(fileframe,JUSTIFY_LEFT|LAYOUT_FILL_X|FRAME_RAISED,0,0,0,0, 3,3,0,0);
+	statusbar=new FXHorizontalFrame(filebox,JUSTIFY_LEFT|LAYOUT_FILL_X|FRAME_RAISED,0,0,0,0, 3,3,0,0);
 	
 	// Read and set sort function for file list
 	FXString sort_func=getApp()->reg().readStringEntry("OPTIONS","sort_func","ascendingCase");
@@ -354,62 +370,82 @@ XFileImage::XFileImage(FXApp* a):FXMainWindow(a,"Xfi ",NULL,NULL,DECOR_ALL)
 		filelist->setDefaultCursor(getApp()->getDefaultCursor(DEF_HAND_CURSOR));
 
     // Status bar buttons
-    new FXToggleButton(statusbar,_("\tShow hidden files (Ctrl-F6)"),_("\tHide hidden files (Ctrl-F6)"),showhiddenicon,hidehiddenicon,this->filelist,
+  	key=getApp()->reg().readStringEntry("KEYBINDINGS","hidden_files","Ctrl-F6");
+    new FXToggleButton(statusbar,TAB+_("Show hidden files")+PARS(key),TAB+_("Hide hidden files")+PARS(key),showhiddenicon,hidehiddenicon,this->filelist,
 	                   FileList::ID_TOGGLE_HIDDEN,BUTTON_TOOLBAR|LAYOUT_LEFT|ICON_BEFORE_TEXT);
-    new FXToggleButton(statusbar,_("\tShow thumbnails (Ctrl-F7)"),_("\tHide thumbnails (Ctrl-F7)"),showthumbicon,hidethumbicon,this->filelist,
+    
+  	key=getApp()->reg().readStringEntry("KEYBINDINGS","thumbnails","Ctrl-F7");
+	new FXToggleButton(statusbar,TAB+_("Show thumbnails")+PARS(key),TAB+_("Hide thumbnails")+PARS(key),showthumbicon,hidethumbicon,this->filelist,
 	                   FileList::ID_TOGGLE_THUMBNAILS,BUTTON_TOOLBAR|LAYOUT_LEFT|ICON_BEFORE_TEXT);
+	
 	new FXStatusBar(statusbar,LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X);
 	new FXDragCorner(statusbar);
 
     // Toolbar button: Open file
-    new FXButton(toolbar,_("\tOpen Image (Ctrl-O)\tOpen image file. (Ctrl-O)"),fileopenicon,this,ID_OPEN,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED);
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","open","Ctrl-O");
+    new FXButton(toolbar,TAB+_("Open")+PARS(key)+TAB+_("Open image file.")+PARS(key),fileopenicon,this,ID_OPEN,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED);
 
     // Toolbar button: Print
-	new FXButton(toolbar,_("\tPrint image (Ctrl-P)\tPrint image file. (Ctrl-P)"),printicon,this,ID_PRINT,BUTTON_TOOLBAR|FRAME_RAISED);
-
-    // Toolbar button: Quit
-    btn=new FXButton(toolbar,_("\tQuit Xfi (Ctrl-Q)\tQuit Xfi. (Ctrl-Q)"),quiticon,this,ID_QUIT,BUTTON_TOOLBAR|FRAME_RAISED);
-    btn->hide();
-    hotkey=(CONTROLMASK<<16) | KEY_w;
-    btn->addHotKey(hotkey);
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","print","Ctrl-P");
+	new FXButton(toolbar,TAB+_("Print")+PARS(key)+TAB+_("Print image file.")+PARS(key),printicon,this,ID_PRINT,BUTTON_TOOLBAR|FRAME_RAISED);
 
 	// Separator
 	toolbarSeparator(toolbar);
 
-    // Toolbar button: Zoom in
-    btn=new FXButton(toolbar,_("\tZoom in (Ctrl+)\tZoom in image. (Ctrl+)"),zoominicon,this,ID_ZOOM_IN,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED);
+    // Note : Ctrl+ and Ctrl- cannot be changed from the registry! 
+	
+	// Toolbar button: Zoom in
+    btn=new FXButton(toolbar,TAB+_("Zoom in")+PARS("Ctrl+")+TAB+_("Zoom in image.")+PARS("Ctrl+"),zoominicon,this,ID_ZOOM_IN,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED);
     hotkey=(CONTROLMASK<<16) |  KEY_KP_Add;
     btn->addHotKey(hotkey);
 
     // Toolbar button: Zoom out
-    btn=new FXButton(toolbar,_("\tZoom out (Ctrl-)\tZoom out image. (Ctrl-)"),zoomouticon,this,ID_ZOOM_OUT,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED);
+    btn=new FXButton(toolbar,TAB+_("Zoom out")+PARS("Ctrl-")+TAB+_("Zoom out image.")+PARS("Ctrl-"),zoomouticon,this,ID_ZOOM_OUT,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED);
     hotkey=(CONTROLMASK<<16) | KEY_KP_Subtract;
     btn->addHotKey(hotkey);
 
     // Toolbar button: Zoom 100%
-    new FXButton(toolbar,_("\tZoom 100% (Ctrl-I)\tZoom 100% image. (Ctrl-I)"),zoom100icon,this,ID_ZOOM_100,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED);
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","zoom_100","Ctrl-I");
+    new FXButton(toolbar,TAB+_("Zoom 100%")+PARS(key)+TAB+_("Zoom image to 100%.")+PARS(key),zoom100icon,this,ID_ZOOM_100,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED);
 
     // Toolbar button: Zoom to fit window
-    new FXButton(toolbar,_("\tZoom to fit window (Ctrl-W)\tZoom to fit window. (Ctrl-W)"),zoomwinicon,this,ID_ZOOM_WIN,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED);
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","zoom_win","Ctrl-F");
+    new FXButton(toolbar,TAB+_("Zoom to fit")+PARS(key)+TAB+_("Zoom to fit window.")+PARS(key),zoomwinicon,this,ID_ZOOM_WIN,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED);
 
 	// Separator
 	toolbarSeparator(toolbar);
 
     // Toolbar button: Rotate left
-    new FXButton(toolbar,_("\tRotate left (Ctrl-L)\tRotate left image. (Ctrl-L)"),rotatelefticon,this,ID_ROTATE_90,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED);
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","rotate_left","Ctrl-L");
+    new FXButton(toolbar,TAB+_("Rotate left")+PARS(key)+TAB+_("Rotate left image.")+PARS(key),rotatelefticon,this,ID_ROTATE_90,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED);
 
     // Toolbar button: Rotate right
-    new FXButton(toolbar,_("\tRotate right (Ctrl-R)\tRotate right image. (Ctrl-R)"),rotaterighticon,this,ID_ROTATE_270,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED);
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","rotate_right","Ctrl-R");
+    new FXButton(toolbar,TAB+_("Rotate right")+PARS(key)+TAB+_("Rotate right image.")+PARS(key),rotaterighticon,this,ID_ROTATE_270,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED);
 
     // Toolbar button: mirror horizontally
-    new FXButton(toolbar,_("\tMirror horizontally (Ctrl-H)\tMirror image horizontally. (Ctrl-H)"),fliplricon,this,ID_MIRROR_HOR,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED);
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","mirror_horizontally","Ctrl-H");
+    new FXButton(toolbar,TAB+_("Mirror horizontally")+PARS(key)+TAB+_("Mirror image horizontally.")+PARS(key),fliplricon,this,ID_MIRROR_HOR,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED);
 
     // Toolbar button: mirror vertically
-    new FXButton(toolbar,_("\tMirror vertically (Ctrl-V)\tMirror image vertically. (Ctrl-V)"),flipudicon,this,ID_MIRROR_VER,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED);
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","mirror_vertically","Ctrl-V");
+    new FXButton(toolbar,TAB+_("Mirror vertically")+PARS(key)+TAB+_("Mirror image vertically.")+PARS(key),flipudicon,this,ID_MIRROR_VER,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED);
 
     // File Menu entries
-    new FXMenuCommand(filemenu,_("&Open...\tCtrl-O\tOpen image file. (Ctrl-O)"),fileopenicon,this,ID_OPEN);
-    new FXMenuCommand(filemenu,_("&Print image...\tCtrl-P\tPrint image file. (Ctrl-P)"),printicon,this,ID_PRINT);
+	FXMenuCommand* mc = NULL;
+	FXString text;
+
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","open","Ctrl-O");
+	text=_("&Open...")+TABS(key)+_("Open image file.")+PARS(key);
+    mc=new FXMenuCommand(filemenu,text,fileopenicon,this,ID_OPEN);
+ 	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
+   
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","print","Ctrl-P");
+	text=_("&Print...")+TABS(key)+_("Print image file.")+PARS(key);
+	mc=new FXMenuCommand(filemenu,text,printicon,this,ID_PRINT);
+ 	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
 
     // Recent file menu; this automatically hides if there are no files
     FXMenuSeparator* sep1=new FXMenuSeparator(filemenu);
@@ -420,45 +456,120 @@ XFileImage::XFileImage(FXApp* a):FXMainWindow(a,"Xfi ",NULL,NULL,DECOR_ALL)
     new FXMenuCommand(filemenu,FXString::null,NULL,&mrufiles,FXRecentFiles::ID_FILE_3);
     new FXMenuCommand(filemenu,FXString::null,NULL,&mrufiles,FXRecentFiles::ID_FILE_4);
     new FXMenuCommand(filemenu,FXString::null,NULL,&mrufiles,FXRecentFiles::ID_FILE_5);
-    new FXMenuCommand(filemenu,_("&Clear recent files\t\tClear recent files."),NULL,&mrufiles,FXRecentFiles::ID_CLEAR);
+    new FXMenuCommand(filemenu,_("&Clear recent files")+TAB2+_("Clear recent file menu."),NULL,&mrufiles,FXRecentFiles::ID_CLEAR);
     FXMenuSeparator* sep2=new FXMenuSeparator(filemenu);
     sep2->setTarget(&mrufiles);
     sep2->setSelector(FXRecentFiles::ID_ANYFILES);
-    new FXMenuCommand(filemenu,_("&Quit\tCtrl-Q\tQuit Xfi. (Ctrl-Q)"),quiticon,this,ID_QUIT);
+
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","quit","Ctrl-Q");
+	text=_("&Quit")+TABS(key)+_("Quit Xfi.")+PARS(key);
+    mc=new FXMenuCommand(filemenu,text,quiticon,this,ID_QUIT);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
 
     // Image Menu entries
-    new FXMenuCommand(imagemenu,_("Zoom &in\tCtrl+\tZoom in image. (Ctrl+)"),zoominicon,this,ID_ZOOM_IN);
-    new FXMenuCommand(imagemenu,_("Zoom &out\tCtrl-\tZoom out image. (Ctrl-)"),zoomouticon,this,ID_ZOOM_OUT);
-    new FXMenuCommand(imagemenu,_("Zoo&m 100%\tCtrl-I\tZoom image to 100%. (Ctrl-I)"),zoom100icon,this,ID_ZOOM_100);
-    new FXMenuCommand(imagemenu,_("Zoom to fit &window\tCtrl-F\tZoom to fit window. (Ctrl-F)"),zoomwinicon,this,ID_ZOOM_WIN);
-    new FXMenuCommand(imagemenu,_("Rotate &right\tCtrl-R\tRotate right. (Ctrl-R)"),rotaterighticon,this,ID_ROTATE_270);
-    new FXMenuCommand(imagemenu,_("Rotate &left\tCtrl-L\tRotate left. (Ctrl-L)"),rotatelefticon,this,ID_ROTATE_90);
-    new FXMenuCommand(imagemenu,_("Mirror &horizontally\tCtrl-H\tMirror horizontally. (Ctrl-H)"),fliplricon,this,ID_MIRROR_HOR);
-    new FXMenuCommand(imagemenu,_("Mirror &vertically\tCtrl-V\tMirror vertically. (Ctrl-V)"),flipudicon,this,ID_MIRROR_VER);
+    new FXMenuCommand(imagemenu,_("Zoom &in")+TAB+(FXString)"Ctrl+"+TAB+_("Zoom in image.")+PARS("Ctrl+"),zoominicon,this,ID_ZOOM_IN);
+	new FXMenuCommand(imagemenu,_("Zoom &out")+TAB+(FXString)"Ctrl-"+TAB+_("Zoom out image.")+PARS("Ctrl-"),zoomouticon,this,ID_ZOOM_OUT);
+	
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","zoom_100","Ctrl-I");
+	text=_("Zoo&m 100%")+TABS(key)+_("Zoom image to 100%.")+PARS(key);
+	mc=new FXMenuCommand(imagemenu,text,zoom100icon,this,ID_ZOOM_100);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
+
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","zoom_win","Ctrl-F");
+	text=_("Zoom to fit &window")+TABS(key)+_("Zoom to fit window.")+PARS(key);
+    mc=new FXMenuCommand(imagemenu,text,zoomwinicon,this,ID_ZOOM_WIN);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
+
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","rotate_right","Ctrl-R");
+	text=_("Rotate &right")+TABS(key)+_("Rotate right.")+PARS(key);
+	mc=new FXMenuCommand(imagemenu,text,rotaterighticon,this,ID_ROTATE_270);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
+
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","rotate_left","Ctrl-L");
+	text=_("Rotate &left")+TABS(key)+_("Rotate left.")+PARS(key);
+    mc=new FXMenuCommand(imagemenu,text,rotatelefticon,this,ID_ROTATE_90);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
+
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","rotate_horizontally","Ctrl-H");
+	text=_("Mirror &horizontally")+TABS(key)+_("Mirror horizontally.")+PARS(key);
+    mc=new FXMenuCommand(imagemenu,text,fliplricon,this,ID_MIRROR_HOR);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
+
+ 	key=getApp()->reg().readStringEntry("KEYBINDINGS","rotate_vertically","Ctrl-V");
+	text=_("Mirror &vertically")+TABS(key)+_("Mirror vertically.")+PARS(key);
+	mc=new FXMenuCommand(imagemenu,text,flipudicon,this,ID_MIRROR_VER);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
 
     // Preferences menu
-    new FXMenuCheck(prefsmenu,_("&Toolbar\t\tDisplay toolbar."),toolbar,FXWindow::ID_TOGGLESHOWN);
-    new FXMenuCheck(prefsmenu,_("&File list\t\tDisplay file list."),filebox,FXWindow::ID_TOGGLESHOWN);
-    new FXMenuCheck(prefsmenu,_("Fit &window when opening\t\tZoom to fit window when opening an image."),this,ID_TOGGLE_FIT_WIN);
+    new FXMenuCheck(prefsmenu,_("&Toolbar")+TAB2+_("Display toolbar."),toolbar,FXWindow::ID_TOGGLESHOWN);
+    new FXMenuCheck(prefsmenu,_("&File list")+TAB2+_("Display file list."),filebox,FXWindow::ID_TOGGLESHOWN);
+    new FXMenuCheck(prefsmenu,_("&Filter images")+TAB2+_("List only image files."),this,ID_TOGGLE_FILTER_IMAGES);
+    new FXMenuCheck(prefsmenu,_("Fit &window when opening")+TAB2+_("Zoom to fit window when opening an image."),this,ID_TOGGLE_FIT_WIN);
 
     // View Menu entries
-    new FXMenuCheck(viewmenu,_("&Hidden files\tCtrl-F6\tShow hidden files and directories. (Ctrl-F6)"),this,ID_TOGGLE_HIDDEN);
-    new FXMenuCheck(viewmenu,_("&Thumbnails\tCtrl-F7\tShow image thumbnails. (Ctrl-F7)"),this,ID_TOGGLE_THUMBNAILS);
+ 	key=getApp()->reg().readStringEntry("KEYBINDINGS","hidden_files","Ctrl-F6");
+	text=_("&Hidden files")+TABS(key)+_("Show hidden files and directories.")+PARS(key);
+    mc=new FXMenuCheck(viewmenu,text,this,ID_TOGGLE_HIDDEN);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
+
+ 	key=getApp()->reg().readStringEntry("KEYBINDINGS","thumbnails","Ctrl-F7");
+	text=_("&Thumbnails")+TABS(key)+_("Show image thumbnails.")+PARS(key);
+    mc=new FXMenuCheck(viewmenu,text,this,ID_TOGGLE_THUMBNAILS);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
+
     new FXMenuSeparator(viewmenu);
-    new FXMenuRadio(viewmenu,_("&Big icons\tF10\tDisplay folders with big icons (F10)."),this,ID_SHOW_BIG);
-    new FXMenuRadio(viewmenu,_("&Small icons\tF11\tDisplay folders with small icons (F11)."),this,ID_SHOW_MINI);
-    new FXMenuRadio(viewmenu,_("&Full file list\tF12\tDisplay detailed folder listing (F12)."),this,ID_SHOW_DETAILS);
+ 
+ 	key=getApp()->reg().readStringEntry("KEYBINDINGS","big_icons","F10");
+	text=_("&Big icons")+TABS(key)+_("Display folders with big icons.")+PARS(key);
+    mc=new FXMenuRadio(viewmenu,text,this,ID_SHOW_BIG);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
+ 
+ 	key=getApp()->reg().readStringEntry("KEYBINDINGS","small_icons","F11");
+	text=_("&Small icons")+TABS(key)+_("Display folders with small icons.")+PARS(key);
+    mc=new FXMenuRadio(viewmenu,text,this,ID_SHOW_MINI);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
+
+ 	key=getApp()->reg().readStringEntry("KEYBINDINGS","detailed_file_list","F12");
+	text=_("&Detailed file list")+TABS(key)+_("Display detailed folder listing.")+PARS(key);
+    mc=new FXMenuRadio(viewmenu,text,this,ID_SHOW_DETAILS);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
+
     new FXMenuSeparator(viewmenu);
-    new FXMenuRadio(viewmenu,_("&Rows\t\tView icons row-wise."),filelist,FileList::ID_ARRANGE_BY_ROWS);
-    new FXMenuRadio(viewmenu,_("&Columns\t\tView icons column-wise."),filelist,FileList::ID_ARRANGE_BY_COLUMNS);
-    new FXMenuCheck(viewmenu,_("&Autosize\t\tAutosize icon names."),filelist,FileList::ID_AUTOSIZE);
+    new FXMenuRadio(viewmenu,_("&Rows")+TAB2+_("View icons row-wise."),filelist,FileList::ID_ARRANGE_BY_ROWS);
+    new FXMenuRadio(viewmenu,_("&Columns")+TAB2+_("View icons column-wise."),filelist,FileList::ID_ARRANGE_BY_COLUMNS);
+    new FXMenuCheck(viewmenu,_("&Autosize")+TAB2+_("Autosize icon names."),filelist,FileList::ID_AUTOSIZE);
 
     // Help Menu entries
-    new FXMenuCommand(helpmenu,_("&About X File Image\tF1\tAbout X File Image (F1)."),NULL,this,ID_ABOUT,0);
+ 	key=getApp()->reg().readStringEntry("KEYBINDINGS","help","F1");
+	text=_("&About X File Image")+TABS(key)+_("About X File Image.")+PARS(key);
+    mc=new FXMenuCommand(helpmenu,text,NULL,this,ID_ABOUT,0);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
+
+	// Close accelerator
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","close","Ctrl-W");
+	hotkey=_parseAccel(key);	
+	getAccelTable()->addAccel(hotkey,this,FXSEL(SEL_COMMAND,XFileImage::ID_QUIT));
 
     // Make a tool tip
     new FXToolTip(getApp(),TOOLTIP_NORMAL);
 	
+	// Images
+	img=NULL;
+	tmpimg=NULL;
+	
 	// Dialogs
 	printdialog=NULL;
 
@@ -472,12 +583,15 @@ XFileImage::XFileImage(FXApp* a):FXMainWindow(a,"Xfi ",NULL,NULL,DECOR_ALL)
 	// Initialize some flags
 	fileview=ID_SHOW_MINI;
 	hiddenfiles=FALSE;	
-	thumbnails=FALSE;	
 	
 	// Initialize zoom to 100%
 	indZoom=ZOOM_100;
 	zoomval=zoomtab[indZoom];
 	fitwin=FALSE;
+	filterimgs=FALSE;
+
+	// Initialize previous window height
+	prev_height=getHeight();
 }
 
 
@@ -494,6 +608,7 @@ XFileImage::~XFileImage()
     delete viewmenu;
     delete dragshell1;
 	delete pathlink;
+	delete pathtext;
 	delete filelist;
 	delete img;
 	delete tmpimg;
@@ -552,6 +667,7 @@ long XFileImage::onCmdDirUp(FXObject*,FXSelector,void*)
 {
     filelist->setDirectory(FXPath::upLevel(filelist->getDirectory()));
    	pathlink->setPath(filelist->getDirectory());
+   	pathtext->setText(filelist->getDirectory());
     return 1;
 }
 
@@ -590,6 +706,7 @@ long  XFileImage::onCmdDirBack(FXObject*,FXSelector s,void* p)
 	// Go to to the previous directory
 	filelist->setDirectory(pathname,FALSE);
    	pathlink->setPath(filelist->getDirectory());
+   	pathtext->setText(filelist->getDirectory());
 
     return 1;
 }
@@ -637,6 +754,7 @@ long  XFileImage::onCmdDirForward(FXObject*,FXSelector s,void* p)
 	// Go to to the previous directory
 	filelist->setDirectory(pathname,FALSE);
    	pathlink->setPath(filelist->getDirectory());
+   	pathtext->setText(filelist->getDirectory());
 
     return 1;
 }
@@ -720,6 +838,8 @@ long  XFileImage::onCmdDirBackHist(FXObject *sender,FXSelector sel,void* ptr)
 			pathname=dirs[pos];
 			filelist->setDirectory(pathname,FALSE);
    			pathlink->setPath(filelist->getDirectory());
+		   	pathtext->setText(filelist->getDirectory());
+
 		}
 		delete[]dirs;
 	}
@@ -805,6 +925,8 @@ long  XFileImage::onCmdDirForwardHist(FXObject *sender,FXSelector sel,void* ptr)
 			pathname=dirs[pos];
 			filelist->setDirectory(pathname,FALSE);
    			pathlink->setPath(filelist->getDirectory());
+		   	pathtext->setText(filelist->getDirectory());
+
 		}
 		delete[]dirs;
 	}
@@ -837,6 +959,7 @@ long XFileImage::onCmdHome(FXObject*,FXSelector,void*)
 {
     filelist->setDirectory(FXSystem::getHomeDirectory());
    	pathlink->setPath(filelist->getDirectory());
+   	pathtext->setText(filelist->getDirectory());
     return 1;
 }
 
@@ -846,6 +969,7 @@ long XFileImage::onCmdWork(FXObject*,FXSelector,void*)
 {
     filelist->setDirectory(FXSystem::getCurrentDirectory());
    	pathlink->setPath(filelist->getDirectory());
+   	pathtext->setText(filelist->getDirectory());
     return 1;
 }
 
@@ -853,9 +977,11 @@ long XFileImage::onCmdWork(FXObject*,FXSelector,void*)
 // About box
 long XFileImage::onCmdAbout(FXObject*,FXSelector,void*)
 {
-    FXString msg=_("X File Image Version ");
-	msg = msg + VERSION + _(" is a simple image viewer.\n\nCopyright (C) 2002-2008 Roland Baudin (roland65@free.fr)");
-    MessageBox about(this,_("About X File Image"),msg.text(),xfiicon,BOX_OK|DECOR_TITLE|DECOR_BORDER);
+	FXString msg;
+	msg.format(_("X File Image Version %s is a simple image viewer.\n\n"),VERSION);
+	msg += COPYRIGHT;
+	MessageBox about(this,_("About X File Image"),msg.text(),xfiicon,BOX_OK|DECOR_TITLE|DECOR_BORDER,
+	                 JUSTIFY_CENTER_X|ICON_BEFORE_TEXT|LAYOUT_TOP|LAYOUT_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y);
     about.execute(PLACEMENT_OWNER);
 
     return 1;
@@ -887,6 +1013,12 @@ FXbool XFileImage::loadimage(const FXString& file)
 			fclose(fp);
 	}
 
+	// Free old image if any, before loading a new one
+	if (img)
+		delete img;
+	if (tmpimg)
+		delete tmpimg;
+
     if(comparecase(ext,"gif")==0)
     {
         img=new FXGIFImage(getApp(),NULL,IMAGE_KEEP|IMAGE_SHMI|IMAGE_SHMP);
@@ -996,6 +1128,7 @@ FXbool XFileImage::loadimage(const FXString& file)
     }
     filelist->setDirectory(FXPath::directory(file));
   	pathlink->setPath(filelist->getDirectory());
+   	pathtext->setText(filelist->getDirectory());
 
     return TRUE;
 }
@@ -1020,6 +1153,47 @@ long XFileImage::onUpdToggleFitWin(FXObject* sender,FXSelector,void*)
 }
 
 
+// Toggle filter image files
+long XFileImage::onCmdToggleFilterImages(FXObject*,FXSelector,void*)
+{
+	filterimgs=!filterimgs;
+	if (filterimgs)
+		filelist->setPattern(imgpatterns);
+	else
+		filelist->setPattern("*");
+
+    return 1;
+}
+
+
+// Update filter image files
+long XFileImage::onUpdToggleFilterImages(FXObject* sender,FXSelector,void*)
+{
+    // Disable menu item if the file list is not shown
+	if(filebox->shown())
+	{
+        sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_ENABLE),NULL);
+		
+		// Update menu item
+		if(filterimgs)
+			sender->handle(this,FXSEL(SEL_COMMAND,ID_CHECK),NULL);
+		else
+			sender->handle(this,FXSEL(SEL_COMMAND,ID_UNCHECK),NULL);
+	}	
+    else
+	{
+        sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_DISABLE),NULL);
+		
+		// Update menu item
+		if(filterimgs)
+			sender->handle(this,FXSEL(SEL_COMMAND,ID_CHECK),NULL);
+		else
+			sender->handle(this,FXSEL(SEL_COMMAND,ID_UNCHECK),NULL);
+	}
+    return 1;
+}
+
+
 // Open
 long XFileImage::onCmdOpen(FXObject*,FXSelector,void*)
 {
@@ -1152,17 +1326,7 @@ long XFileImage::onCmdShowDetails(FXObject*,FXSelector,void*)
 long XFileImage::onUpdFileView(FXObject* sender,FXSelector sel,void* ptr)
 {
 	// Keep the filebox height relative to the window height
-	
-	static FXint flag=0;
-	static FXint prev_height;
 
-	// Initialize the previous window height
-	if (flag==0)
-	{
-		prev_height=getHeight();
-		flag=1;
-	}
-	
 	// Get the current height
 	FXint height=getHeight();
 	
@@ -1216,6 +1380,14 @@ long XFileImage::onUpdFileView(FXObject* sender,FXSelector sel,void* ptr)
 }
 
 
+// Harvest the zombies
+long XFileImage::onSigHarvest(FXObject*,FXSelector,void*)
+{
+	while(waitpid(-1,NULL,WNOHANG)>0);
+	return 1;
+}
+
+
 // Quit
 long XFileImage::onCmdQuit(FXObject*,FXSelector,void*)
 {
@@ -1268,6 +1440,7 @@ long XFileImage::onCmdItemDoubleClicked(FXObject*,FXSelector,void* ptr)
 			}
             filelist->setDirectory(filepath);
             pathlink->setPath(filepath);
+   			pathtext->setText(filepath);
         }
         else if(filelist->isItemFile(index))
         {
@@ -1308,6 +1481,7 @@ long XFileImage::onCmdItemClicked(FXObject* sender,FXSelector sel, void* ptr)
 				}
 				filelist->setDirectory(filepath);
 				pathlink->setPath(filepath);
+   				pathtext->setText(filepath);
 			}
 			else if((single_click == SINGLE_CLICK_DIR_FILE) && filelist->isItemFile(index) && allow)
 			{
@@ -1555,19 +1729,24 @@ void XFileImage::create()
 	// Get size and position
     FXuint ww=getApp()->reg().readUnsignedEntry("OPTIONS","width",DEFAULT_WINDOW_WIDTH);    // Workaround for a possible bug in some WMs
     FXuint hh=getApp()->reg().readUnsignedEntry("OPTIONS","height",DEFAULT_WINDOW_HEIGHT);  // Workaround for a possible bug in some WMs
-    FXint xx=getApp()->reg().readIntEntry("OPTIONS","xpos",DEFAULT_WINDOW_XPOS);
-    FXint yy=getApp()->reg().readIntEntry("OPTIONS","ypos",DEFAULT_WINDOW_YPOS);
     fileheight_pct=getApp()->reg().readRealEntry("OPTIONS","fileheight_pct",0.25);
     FXuint fs=getApp()->reg().readIntEntry("OPTIONS","filesshown",TRUE);
 
 	filelist->setDirectory(FXSystem::getCurrentDirectory());
    	pathlink->setPath(FXSystem::getCurrentDirectory());
+   	pathtext->setText(FXSystem::getCurrentDirectory());
 
-	// Display or hide path linker
+	// Display or hide path linker and path text
 	if (show_pathlink)
+	{
+		pathtext->hide();
 		pathlink->show();
+	}
 	else
+	{
 		pathlink->hide();
+		pathtext->show();
+	}
 
     // Hide tree if asked for
     if(!fs)
@@ -1586,8 +1765,8 @@ void XFileImage::create()
 	filelist->showHiddenFiles(hiddenfiles);
   
  	// Get thumbnails status
-	thumbnails=getApp()->reg().readUnsignedEntry("OPTIONS","thumbnails",0);
-	filelist->showThumbnails(thumbnails);
+	//thumbnails=getApp()->reg().readUnsignedEntry("OPTIONS","thumbnails",0);
+	//filelist->showThumbnails(thumbnails);
   
 	// Get list style
 	liststyle=getApp()->reg().readUnsignedEntry("OPTIONS","liststyle",(FXuint)IconList::ID_SHOW_DETAILS);
@@ -1600,15 +1779,32 @@ void XFileImage::create()
 	// Get startup zoom
 	fitwin=getApp()->reg().readUnsignedEntry("OPTIONS","fitwin",0);
 	
-	// Reposition window
-    position(xx,yy,ww,hh);
+	// Get filter images flag
+	filterimgs=getApp()->reg().readUnsignedEntry("OPTIONS","filterimgs",FALSE);
+	
+	// Get position and position window
+    if (save_win_pos)
+	{
+		FXint xpos=getApp()->reg().readIntEntry("OPTIONS","xpos",DEFAULT_WINDOW_XPOS);
+		FXint ypos=getApp()->reg().readIntEntry("OPTIONS","ypos",DEFAULT_WINDOW_YPOS);
+		position(xpos,ypos,ww,hh);
+	}
+	else
+    	position(getX(),getY(),ww,hh);
 
     FXMainWindow::create();
 
+	if (filterimgs)
+		filelist->setPattern(imgpatterns);
+
 	// Set file list height
     filebox->setHeight((FXint)round(fileheight_pct*getHeight()));
 
 	show();
+
+#ifdef STARTUP_NOTIFICATION
+	startup_completed();
+#endif
 }
 
 
@@ -1618,8 +1814,12 @@ void XFileImage::saveConfig()
     // Write new window size and position back to registry
 	getApp()->reg().writeUnsignedEntry("OPTIONS","width",(FXuint)getWidth());
 	getApp()->reg().writeUnsignedEntry("OPTIONS","height",(FXuint)getHeight());
-	getApp()->reg().writeIntEntry("OPTIONS","xpos",getX());
-	getApp()->reg().writeIntEntry("OPTIONS","ypos",getY());
+
+	if (save_win_pos)
+	{
+		getApp()->reg().writeIntEntry("OPTIONS","xpos",getX());
+		getApp()->reg().writeIntEntry("OPTIONS","ypos",getY());
+	}
 
     // Width of tree
     getApp()->reg().writeRealEntry("OPTIONS","fileheight_pct",fileheight_pct);
@@ -1648,6 +1848,9 @@ void XFileImage::saveConfig()
 	// Startup zoom
 	getApp()->reg().writeUnsignedEntry("OPTIONS","fitwin",fitwin);
 
+	// Filter images in file list
+	getApp()->reg().writeUnsignedEntry("OPTIONS","filterimgs",filterimgs);
+
 	// Filelist columns sizes
 	getApp()->reg().writeUnsignedEntry("OPTIONS","name_size",filelist->getHeaderSize(0));
 	getApp()->reg().writeUnsignedEntry("OPTIONS","size_size",filelist->getHeaderSize(1));
@@ -1851,12 +2054,18 @@ int main(int argc,char *argv[])
 	// Set display pathlinker flag according to the Xfe registry
 	show_pathlink=reg_xfe->readUnsignedEntry("SETTINGS","show_pathlinker",TRUE);
 	    
+    // Retain window position flag
+	save_win_pos=reg_xfe->readUnsignedEntry("SETTINGS","save_win_pos",FALSE);
+
 	// Delete the Xfe registry
 	delete reg_xfe;
 
 	// Make window
-    XFileImage* window=new XFileImage(application);
+    XFileImage* window=new XFileImage(application,smoothscroll);
 	
+ 	// Catch SIGCHLD to harvest zombie child processes
+	application->addSignal(SIGCHLD,window,XFileImage::ID_HARVEST,TRUE);
+
 	// Smooth scrolling
 	window->setSmoothScroll(smoothscroll);
 
