@@ -10,14 +10,16 @@
 #include <string.h>
 #include <unistd.h>
 #include <sys/stat.h>
+#include <sys/wait.h>
 
 #include "xfedefs.h"
 #include "icons.h"
-#include "File.h"
+#include "xfeutils.h"
+#include "startupnotification.h"
 #include "FileDialog.h"
 #include "MessageBox.h"
-#include "XFilePackage.h"
 #include "CommandWindow.h"
+#include "XFilePackage.h"
 
 
 // Add FOX hacks
@@ -36,6 +38,7 @@ FXbool allowPopupScroll=FALSE;
 FXuint single_click;
 FXbool file_tooltips;
 FXbool relative_resize;
+FXbool save_win_pos;
 
 // Hand cursor replacement
 #define hand_width 32
@@ -72,6 +75,7 @@ FXbool relative_resize;
 // Map
 FXDEFMAP(XFilePackage) XFilePackageMap[]=
 {
+	FXMAPFUNC(SEL_SIGNAL,XFilePackage::ID_HARVEST,XFilePackage::onSigHarvest),
 	FXMAPFUNC(SEL_CLOSE,0,XFilePackage::onCmdQuit),
 	FXMAPFUNC(SEL_COMMAND,XFilePackage::ID_QUIT,XFilePackage::onCmdQuit),
 	FXMAPFUNC(SEL_COMMAND,XFilePackage::ID_UNINSTALL,XFilePackage::onCmdUninstall),
@@ -89,8 +93,8 @@ FXIMPLEMENT(XFilePackage,FXMainWindow,XFilePackageMap,ARRAYNUMBER(XFilePackageMa
 // Constructor
 XFilePackage::XFilePackage(FXApp *a):FXMainWindow(a,"Xfp",NULL,NULL,DECOR_ALL)
 {
-    FXHotKey hotkey;
-    FXButton *btn = NULL;
+	FXString key;
+	FXHotKey hotkey;
 
     setIcon(xfpicon);
 		
@@ -105,7 +109,7 @@ XFilePackage::XFilePackage(FXApp *a):FXMainWindow(a,"Xfp",NULL,NULL,DECOR_ALL)
 
     // Tool bar
   	FXToolBarShell* dragshell1=new FXToolBarShell(this,FRAME_RAISED);
-  	toolbar=new FXToolBar(topdock,dragshell1,LAYOUT_DOCK_NEXT|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|FRAME_RAISED);
+  	toolbar=new FXToolBar(topdock,dragshell1,LAYOUT_DOCK_NEXT|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_RAISED);
   	new FXToolBarGrip(toolbar,toolbar,FXToolBar::ID_TOOLBARGRIP,TOOLBARGRIP_DOUBLE);
     int style=BUTTON_TOOLBAR;
 
@@ -122,23 +126,38 @@ XFilePackage::XFilePackage(FXApp *a):FXMainWindow(a,"Xfp",NULL,NULL,DECOR_ALL)
     new FXMenuTitle(menubar,_("&Help"),NULL,helpmenu);
 
     // Toolbar button: File open
-    new FXButton(toolbar,_("\tOpen package file (Ctrl-O)"),fileopenicon,this,ID_OPEN,style|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
-
-    // Toolbar button: Quit
-    btn=new FXButton(toolbar,_("\tQuit Xfp (Ctrl-Q)"),quiticon,this,ID_QUIT,style|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
-    btn->hide();
-    hotkey=(CONTROLMASK<<16) | KEY_w;
-    btn->addHotKey(hotkey);
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","open","Ctrl-O");
+    new FXButton(toolbar,TAB+_("Open package file")+PARS(key),fileopenicon,this,ID_OPEN,style|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
 
     // File Menu entries
-    new FXMenuCommand(filemenu,_("&Open file...\tCtrl-O"),fileopenicon,this,ID_OPEN);
-    new FXMenuCommand(filemenu,_("&Quit\tCtrl-Q"),quiticon,this,ID_QUIT);
+	FXMenuCommand* mc = NULL;
 
+    mc=new FXMenuCommand(filemenu,_("&Open..."),fileopenicon,this,ID_OPEN);
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","open","Ctrl-O");
+	mc->setAccelText(key);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
+
+    mc=new FXMenuCommand(filemenu,_("&Quit"),quiticon,this,ID_QUIT);
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","quit","Ctrl-Q");
+	mc->setAccelText(key);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
+	
     // Preferences menu
-    new FXMenuCheck(prefsmenu,_("&Toolbar\t\tDisplay toolbar"),toolbar,FXWindow::ID_TOGGLESHOWN);
+    new FXMenuCheck(prefsmenu,_("&Toolbar")+TAB+_("Display toolbar"),toolbar,FXWindow::ID_TOGGLESHOWN);
 
     // Help Menu entries
-    new FXMenuCommand(helpmenu,_("&About X File Package\tF1"),NULL,this,ID_ABOUT,0);
+    mc=new FXMenuCommand(helpmenu,_("&About X File Package"),NULL,this,ID_ABOUT,0);
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","help","F1");
+	mc->setAccelText(key);
+	hotkey=_parseAccel(key);
+	getAccelTable()->addAccel(hotkey,mc,FXSEL(SEL_COMMAND,FXMenuCommand::ID_ACCEL));
+
+	// Close accelerator
+	key=getApp()->reg().readStringEntry("KEYBINDINGS","close","Ctrl-W");
+	hotkey=_parseAccel(key);	
+	getAccelTable()->addAccel(hotkey,this,FXSEL(SEL_COMMAND,XFilePackage::ID_QUIT));
 
     // Make a tool tip
     new FXToolTip(getApp(),0);
@@ -157,18 +176,18 @@ XFilePackage::XFilePackage(FXApp *a):FXMainWindow(a,"Xfp",NULL,NULL,DECOR_ALL)
     new FXButton(buttons,_("&Install/Upgrade"),NULL,this,XFilePackage::ID_INSTALL,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y,0,0,0,0,20,20);
 
     // Switcher
-    FXTabBook* tabbook = new FXTabBook(this,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_RIGHT);
+    FXTabBook* tabbook = new FXTabBook(this,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_RIGHT|FRAME_RAISED,0,0,0,0, 0,0,0,0);
 
     // First item is Description
     new FXTabItem(tabbook,_("&Description"),NULL);
     FXVerticalFrame *descr=new FXVerticalFrame(tabbook);
-	FXPacker *p=new FXPacker(descr,FRAME_SUNKEN|LAYOUT_FILL_X|LAYOUT_FILL_Y);
+	FXPacker *p=new FXPacker(descr,LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_SUNKEN,0,0,0,0, 0,0,0,0);
     description=new FXText(p,this,XFilePackage::ID_DESCRIPTION,LAYOUT_FILL_X|LAYOUT_FILL_Y);
 
     // Second item is File List
     new FXTabItem(tabbook,_("File &List"),NULL);
     FXVerticalFrame *flistfr=new FXVerticalFrame(tabbook);
-    p=new FXPacker(flistfr,FRAME_SUNKEN|LAYOUT_FILL_X|LAYOUT_FILL_Y);
+    p=new FXPacker(flistfr,LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_SUNKEN,0,0,0,0, 0,0,0,0);
 	list=new FXTreeList(p,this,XFilePackage::ID_FILELIST,LAYOUT_FILL_X|LAYOUT_FILL_Y|TREELIST_SHOWS_LINES);
 
 	// Initialize file name
@@ -190,9 +209,11 @@ XFilePackage::~XFilePackage()
 // About box
 long XFilePackage::onCmdAbout(FXObject*,FXSelector,void*)
 {
-    FXString msg=_("X File Package Version ");
-	msg = msg + VERSION + _(" is a simple rpm or deb package manager.\n\nCopyright (C) 2002-2008 Roland Baudin (roland65@free.fr)");
-	MessageBox about(this,_("About X File Package"),msg.text(),xfpicon,BOX_OK|DECOR_TITLE|DECOR_BORDER);
+	FXString msg;
+	msg.format(_("X File Package Version %s is a simple rpm or deb package manager.\n\n"),VERSION);
+	msg += COPYRIGHT;
+	MessageBox about(this,_("About X File Package"),msg.text(),xfpicon,BOX_OK|DECOR_TITLE|DECOR_BORDER,
+	                 JUSTIFY_CENTER_X|ICON_BEFORE_TEXT|LAYOUT_TOP|LAYOUT_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y);
     about.execute(PLACEMENT_OWNER);
     return 1;
 }
@@ -255,7 +276,8 @@ long XFilePackage::onCmdInstall(FXObject*,FXSelector,void*)
 	CommandWindow *cmdwin=new CommandWindow(this,_("Install/Upgrade Package"),cmd,10,80);			
 	cmdwin->create();
 
-	FXString msg=_("Installing package: ") + package + "\n"; 
+	FXString msg;
+	msg.format(_("Installing package: %s \n"),package.text());
 	cmdwin->appendText(msg.text());
 
 	// The CommandWindow object will delete itself when closed!
@@ -304,7 +326,8 @@ long XFilePackage::onCmdUninstall(FXObject*,FXSelector,void*)
 	CommandWindow *cmdwin=new CommandWindow(this,_("Uninstall Package"),cmd,10,80);			
 	cmdwin->create();
 
-	FXString msg=_("Uninstalling package: ") + package + "\n"; 
+	FXString msg;
+	msg.format(_("Uninstalling package: %s \n"),package.text());	
 	cmdwin->appendText(msg.text());
 
 	// The CommandWindow object will delete itself when closed!
@@ -319,8 +342,12 @@ void XFilePackage::saveConfig()
     // Write new window size and position back to registry
     getApp()->reg().writeUnsignedEntry("OPTIONS","width",(FXuint)getWidth());
     getApp()->reg().writeUnsignedEntry("OPTIONS","height",(FXuint)getHeight());
-	getApp()->reg().writeIntEntry("OPTIONS","xpos",getX());
-	getApp()->reg().writeIntEntry("OPTIONS","ypos",getY());
+
+	if (save_win_pos)
+	{
+		getApp()->reg().writeIntEntry("OPTIONS","xpos",getX());
+		getApp()->reg().writeIntEntry("OPTIONS","ypos",getY());
+	}
 
     // Last opened filename
     getApp()->reg().writeStringEntry("OPTIONS","filename",filename.text());
@@ -331,15 +358,23 @@ void XFilePackage::saveConfig()
     else
         getApp()->reg().writeUnsignedEntry("OPTIONS","showtoolbar",FALSE);
 
-	// Write registry OPTIONS
+	// Write registry options
 	getApp()->reg().write();
 }
 
 
+// Harvest the zombies
+long XFilePackage::onSigHarvest(FXObject*,FXSelector,void*)
+{
+	while(waitpid(-1,NULL,WNOHANG)>0);
+	return 1;
+}
+
+
 // Quit application
 long XFilePackage::onCmdQuit(FXObject*,FXSelector,void*)
 {
-	// Save OPTIONS 
+	// Save options
     saveConfig();
 
     // Exit
@@ -392,10 +427,11 @@ long XFilePackage::onUpdFileList(FXObject*,FXSelector,void*)
     {
         text[strlen(text)-1]='\0';// kill trailing lf
         list->appendItem(topmost,text,minidocicon,minidocicon);
-  }
-    list->expandTree(topmost);
-    pclose(rpmcmd);
-    return 1;
+	}
+	list->expandTree(topmost);
+	
+	pclose(rpmcmd);
+	return 1;
 }
 
 
@@ -521,21 +557,37 @@ void XFilePackage::create()
 	}
 	delete reg_xfe;
 				
-    // Get size
-    FXuint width=getApp()->reg().readUnsignedEntry("OPTIONS","width",DEFAULT_WINDOW_WIDTH);
-    FXuint height=getApp()->reg().readUnsignedEntry("OPTIONS","height",DEFAULT_WINDOW_HEIGHT);
-    FXint xpos=getApp()->reg().readIntEntry("OPTIONS","xpos",DEFAULT_WINDOW_XPOS);
-    FXint ypos=getApp()->reg().readIntEntry("OPTIONS","ypos",DEFAULT_WINDOW_YPOS);
-
 	// Get toolbar status
     if(getApp()->reg().readUnsignedEntry("OPTIONS","showtoolbar",TRUE)==FALSE)
         toolbar->hide();
 
-    // Reposition window
-    position(xpos,ypos,width,height);
+    // Get size
+    FXuint width=getApp()->reg().readUnsignedEntry("OPTIONS","width",DEFAULT_WINDOW_WIDTH);
+    FXuint height=getApp()->reg().readUnsignedEntry("OPTIONS","height",DEFAULT_WINDOW_HEIGHT);
+
+	// Get position and position window
+    if (save_win_pos)
+	{
+		FXint xpos=getApp()->reg().readIntEntry("OPTIONS","xpos",DEFAULT_WINDOW_XPOS);
+		FXint ypos=getApp()->reg().readIntEntry("OPTIONS","ypos",DEFAULT_WINDOW_YPOS);
+		position(xpos,ypos,width,height);
+	}
+	else
+    	position(getX(),getY(),width,height);
+
     FXMainWindow::create();
+
+	// This is necessary otherwise Xfp crashes when opening a package, but why?
+	minifoldericon->create();
+	minifolderopenicon->create();
+	minidocicon->create();
+
     show();
     description->handle(this,FXSEL(SEL_COMMAND,FXText::ID_TOGGLE_EDITABLE),NULL);
+
+#ifdef STARTUP_NOTIFICATION
+	startup_completed();
+#endif
 }
 
 
@@ -575,8 +627,6 @@ int main(int argc,char* argv[])
 	bind_textdomain_codeset(PACKAGE,"utf-8");
 	textdomain (PACKAGE);
 #endif
-
-	XFilePackage *window;
 	
 	// Parse basic arguments
 	for(i = 1; i < argc; ++i)
@@ -604,7 +654,7 @@ int main(int argc,char* argv[])
     // Make application
     FXApp* application=new FXApp(appname,vdrname);
 
-    // Open display
+	// Open display
     application->init(argc,argv);
 
     // Redefine the default hand cursor
@@ -645,11 +695,17 @@ int main(int argc,char* argv[])
 	// Set relative resizing flag according to the Xfe registry
 	relative_resize=reg_xfe->readUnsignedEntry("SETTINGS","relative_resize",TRUE);
 	
+    // Retain window position flag
+	save_win_pos=reg_xfe->readUnsignedEntry("SETTINGS","save_win_pos",FALSE);
+
 	// Delete the Xfe registry
 	delete reg_xfe;
 
 	// Make window
-	window=new XFilePackage(application);
+	XFilePackage* window=new XFilePackage(application);
+
+ 	// Catch SIGCHLD to harvest zombie child processes
+	application->addSignal(SIGCHLD,window,XFilePackage::ID_HARVEST,TRUE);
 
 	// Smooth scrolling
 	window->setSmoothScroll(smoothscroll);
