@@ -30,12 +30,13 @@ public:
         ID_INSTALL,
 		ID_ABOUT,
 		ID_OPEN,
+        ID_HARVEST,
         ID_QUIT,
         ID_LAST
     };
-    void start(FXString startpkg);
+    void start(FXString);
     void create();
-    XFilePackage(FXApp *a);
+    XFilePackage(FXApp*);
 	~XFilePackage();
 	void setSmoothScroll(FXbool smooth)
 	{
@@ -48,6 +49,7 @@ public:
     long onUpdDescription(FXObject*,FXSelector,void*);
     long onUpdFileList(FXObject*,FXSelector,void*);
 	void saveConfig();
+	long onSigHarvest(FXObject*,FXSelector,void*);
     long onCmdQuit(FXObject*,FXSelector,void*);
 };
 
