@@ -27,7 +27,7 @@ public:
 public:
 
     // Construct application object
-    XFileView(const FXString& appname, const FXString& vdrname);
+    XFileView(const FXString&, const FXString&);
 
     // Initialize application
     virtual void init(int& argc,char** argv,bool connect=TRUE);
