@@ -17,9 +17,10 @@
 
 #include "xfedefs.h"
 #include "icons.h"
+#include "xfeutils.h"
+#include "startupnotification.h"
 #include "MessageBox.h"
 #include "EditWindow.h"
-#include "File.h"
 #include "XFileWrite.h"
 
 // Add FOX hacks
@@ -232,6 +233,10 @@ int main(int argc,char *argv[])
         {
             file=FXPath::absolute(argv[i]);
             window=new EditWindow(application,_("untitled"));
+
+			// Catch SIGCHLD to harvest zombie child processes
+			application->addSignal(SIGCHLD,window,EditWindow::ID_HARVEST,TRUE);
+
  			window->setSmoothScroll(smoothscroll);
 			window->create();
             window->loadFile(file);
@@ -242,6 +247,10 @@ int main(int argc,char *argv[])
     if(!window)
     {
         window=new EditWindow(application,_("untitled"));
+
+		// Catch SIGCHLD to harvest zombie child processes
+		application->addSignal(SIGCHLD,window,EditWindow::ID_HARVEST,TRUE);
+
 		window->setSmoothScroll(smoothscroll);
         window->create();
     }
