@@ -28,10 +28,10 @@ public:
 public:
 
     // Construct application object
-    XFileWrite(const FXString& appname, const FXString& vdrname);
+    XFileWrite(const FXString&, const FXString&);
 
     // Initialize application
-    virtual void init(int& argc,char** argv,bool connect=TRUE);
+    virtual void init(int& argc, char** argv,bool connect=TRUE);
 
     // Exit application
     virtual void exit(FXint code=0);
