@@ -5,6 +5,7 @@
 
 #include <fx.h>
 #include <FXPNGIcon.h>
+
 #include "xfedefs.h"
 #include "icons.h"
 
@@ -36,7 +37,8 @@ FXIcon *trash_full_bigicon, *trash_fullicon, *treeonepanelicon, *treetwopanelsic
 FXIcon *undoicon, *unmaphosticon, *uppercaseicon, *warningbigicon, *workicon, *wrapofficon, *wraponicon, *xfeicon, *xfiicon;
 FXIcon *xfpicon, *xfvicon, *xfwicon, *zipicon, *zoom100icon, *zoominicon, *zoomouticon, *zoomwinicon;
 FXIcon *totrashicon, *dirbackicon, *dirforwardicon, *minixfeicon, *minixferooticon, *filedialogicon, *bigarchaddicon;
-FXIcon *switchpanelsicon, *syncpanelsicon, *newlinkicon;
+FXIcon *switchpanelsicon, *syncpanelsicon, *newlinkicon, *greenbuttonicon, *graybuttonicon;
+FXIcon *keybindingsicon, *minikeybindingsicon, *filerestoreicon, *restore_bigicon;
 
 
 // Load a PNG icon from a file in the icon path
@@ -61,9 +63,6 @@ FXIcon* loadiconfile(FXApp *app, const FXString iconpath, const FXString iconnam
 				// Load it
 				icon->loadPixels(str);
 
-				// Create it
-				icon->create();
-
 				// Done
 				str.close();
 
@@ -231,6 +230,12 @@ FXbool loadAppIcons(FXApp *app)
 	success=( (switchpanelsicon=loadiconfile(app,iconpath,"switchpanels.png")) != NULL ) & success;
 	success=( (syncpanelsicon=loadiconfile(app,iconpath,"syncpanels.png")) != NULL ) & success;
 	success=( (newlinkicon=loadiconfile(app,iconpath,"newlink.png")) != NULL ) & success;
+	success=( (greenbuttonicon=loadiconfile(app,iconpath,"greenbutton.png")) != NULL ) & success;
+	success=( (graybuttonicon=loadiconfile(app,iconpath,"graybutton.png")) != NULL ) & success;
+	success=( (keybindingsicon=loadiconfile(app,iconpath,"keybindings.png")) != NULL ) & success;
+	success=( (minikeybindingsicon=loadiconfile(app,iconpath,"minikeybindings.png")) != NULL ) & success;
+	success=( (filerestoreicon=loadiconfile(app,iconpath,"filerestore.png")) != NULL ) & success;
+	success=( (restore_bigicon=loadiconfile(app,iconpath,"restore_big.png")) != NULL ) & success;
 		
 	return success;
 }
