@@ -151,5 +151,11 @@ extern FXIcon *bigarchaddicon;
 extern FXIcon *switchpanelsicon;
 extern FXIcon *syncpanelsicon;
 extern FXIcon *newlinkicon;
+extern FXIcon *greenbuttonicon;
+extern FXIcon *graybuttonicon;
+extern FXIcon *keybindingsicon;
+extern FXIcon *minikeybindingsicon;
+extern FXIcon *filerestoreicon;
+extern FXIcon *restore_bigicon;
 #endif
 
